﻿using System;
using System.Runtime.InteropServices;
using System.Text;

namespace Nirva
{
    public interface nvsint
    {
        void SetError(String ErrorInfo);
        void SetErrorEx(String ErrorService, String ErrorClass, int ErrorCode, String ErrorInfo);
        int Command(String CommandString);
        String GetResult();
        String GetSessionId();
        String GetService();
        String GetClass();
        String GetCommand();
        String GetInContainer();
        String GetOutContainer();
        String GetLanguage();
        String GetError(String InfoType);
        int GetNumParameters();
        String GetSource();
        bool ParameterExist(String ParameterName);
        String GetParameter(String ParameterName);
        String GetParameterName(int Index);
    }
    public interface nvcmd
    {
        void SetError(String ErrorClass, int ErrorCode, String ErrorInfo);
        bool Command(String CommandString);
        String GetResult();
        String GetSessionId();
        String GetClass();
        String GetCommand();
        String GetInContainer();
        String GetOutContainer();
        String GetMachineName();
        String GetMachineUser();
        String GetLanguage();
        String GetError(String InfoType);
        int GetNumParameters();
        String GetSource();
        bool IsCommand(String Class, String Command, String Source);
        bool ParameterExist(String ParameterName);
        String GetParameter(String ParameterName);
        void AddHttpHeader(String headerName, String headerValue);
        String GetParameterName(int Index);
    }
    [Serializable]
    public class nvrend
    {
        public byte[] Input;
        public byte[] Output;
        public String PageName;
        public String InputEncoding;
        public String SessionId;
        public String AppName;
        public String Reason;
        public String ContentType;
        public int InputLength;
             // Default constructor
        public nvrend()
        {
            Input = null;
            Output = null;
            PageName = "";
            SessionId = "";
            AppName = "";
            Reason = "";
            ContentType = "";
            InputEncoding = "";
        }
    }
    public abstract class ProcedureEntryPoint : MarshalByRefObject
    {
        public abstract int Main(nvsint Command);
    }
    public abstract class ServiceEntryPoint : MarshalByRefObject
    {
        public abstract bool Command(nvcmd Command);
        public abstract bool Init(nvcmd Command);
        public abstract bool Exit(nvcmd Command);
        public override object InitializeLifetimeService()
        {
            return null;
        }
        public virtual bool Render(ref nvrend Rend)
        {
            Rend.Reason = "This is not a renderer service";
            return false;
        }
    }
    public interface IServiceSession
    {
        bool OnInit(nvcmd Command);
        bool OnCleanup(nvcmd Command);
        bool OnCommand(nvcmd Command);
    }
    public class nvcdn
    {
        // Import all native dll functions
        [DllImport("nvc.dll", EntryPoint = "NvOpenRequestW", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.StdCall)]
        public static extern ulong NvOpenRequestW([MarshalAs(UnmanagedType.LPWStr)] string ConnectString);
        [DllImport("nvc.dll", EntryPoint = "NvCloseRequestW", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.StdCall)]
        public static extern void NvCloseRequestW(ulong Request);
        [DllImport("nvc.dll", EntryPoint = "NvCommandW", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.StdCall)]
        public static extern uint NvCommandW(ulong Request, [MarshalAs(UnmanagedType.LPWStr)] string Command, [MarshalAs(UnmanagedType.LPWStr)]StringBuilder Buffer, int BufferSize, ref int DataSize);

        private
        ulong MyRequest;
        int BufSize;
        StringBuilder Buffer;

        public
        String NvResult;


            // Default constructor
        public nvcdn()
        {
            // We just force unicode.
            MyRequest = NvOpenRequestW("Unicode=\"YES\"");
            BufSize = 256;
            Buffer = new StringBuilder(BufSize);
            NvResult = "";
        }

        // Constructor with connection string
        public nvcdn(String ConnecString)
        {
            // We use the connection string but we force unicode.
            MyRequest = NvOpenRequestW(ConnecString + "Unicode=\"YES\"");
            BufSize = 256;
            Buffer = new StringBuilder(BufSize);
            NvResult = "";
        }

        public int Command(String CommandString)
        {
            NvResult = "";
            int DataSize = 0;
            uint RetVal = NvCommandW(MyRequest, CommandString, Buffer, BufSize, ref DataSize);
            if (RetVal == 1)
            {
                if (DataSize >= BufSize)
                {
                    // We must create a larger buffer
                    BufSize = DataSize + 1;
                    Buffer = null;
                    Buffer = new StringBuilder(BufSize);
                    // And get the last output buffer
                    RetVal = NvCommandW(MyRequest, "NV_CMD=|LOCAL:GET_LAST_OUTPUT_BUFFER|", Buffer, BufSize, ref DataSize);
 //                   RetVal = NvCommandW(MyRequest, CommandString, Buffer, BufSize, ref DataSize);
                }
            }
            if (RetVal == 1)
                NvResult = Buffer.ToString();

            return (int)RetVal;
        }
        public String GetResult()
        {
            return NvResult;
        }

        // finalization
        ~nvcdn()
        {
            if (MyRequest != 0)
                NvCloseRequestW(MyRequest);
            MyRequest = 0;
        }
    }
}
