// Http.cpp : implementation file
//


#ifdef WIN32
#define WINVER 0x0500
#define _WIN32_WINNT 0x0500
#endif
#include "nvcc.h"

using namespace std;



/////////////////////////////////////////////////////////////////////////////
// NvHttpConnectionList

NvHttpConnectionList::NvHttpConnectionList()
{
}

NvHttpConnectionList::~NvHttpConnectionList()
{
	Clear();
}




void NvHttpConnectionList::Clear()
{
	int i;
	Lock();
	for(i=0; i<(int)m_vConnections.size(); i++)
	{
		NvHttpConnection *Connection = (NvHttpConnection *)m_vConnections[i];
		if(Connection != NULL)
			delete Connection;
	}
	m_vConnections.clear();
	Unlock();
}


NvHttpConnection* NvHttpConnectionList::GetConnection(const char *Address, int Port, int ConnectionTimeOut, bool UseSSL, int ProxyMode, const char *FinalAddress, int FinalPort, const char *Certificate)
{
	// We stop updating list for each connection requesty for performance reasons
	//	Update();

	string AddressHere = "127.0.0.1";
	if(Address != NULL)
		AddressHere = Address;
	CleanString(AddressHere);
	if(!AddressHere.length())
		AddressHere = "127.0.0.1";

	string CertificateHere = "";
	if(Certificate != NULL)
		CertificateHere = Certificate;
	string CertificatePassword = "";
	string::size_type index = CertificateHere.find(';');
	if(index != string::npos)
	{
		CertificatePassword = CertificateHere.substr(index+1);
		CertificateHere = CertificateHere.substr(0, index);
	}

	NvHttpConnection *FoundConnection = NULL;
	
	Lock();
	int i;
	for(i=0; i<(int)m_vConnections.size(); i++)
	{
		NvHttpConnection *Connection = (NvHttpConnection *)m_vConnections[i];
		if(Connection == NULL)
			continue;
		if((AddressHere == Connection->m_sServer) && (Port == Connection->m_iPort) && (UseSSL == Connection->b_UseSSL) && (ProxyMode == Connection->m_iProxyMode) && (CertificateHere == Connection->m_Certificate) && (CertificatePassword == Connection->m_CertificatePassword))
		{
			if(Connection->IsInUse())
				continue;
			if((ProxyMode == PROXY_FINAL_SSL))
			{
				if((Connection->m_sFinalAddress != FinalAddress) || (Connection->m_iFinalPort != FinalPort))
					continue;
			}
			FoundConnection = Connection;
			break;
		}
	}

	if(FoundConnection == NULL)
	{
		NvHttpConnection *Connection = new NvHttpConnection;
		if(Connection == NULL)
		{
			Unlock();
			return NULL;
		}
		Connection->m_sServer = AddressHere;
		Connection->m_sFinalAddress = FinalAddress;
		Connection->m_iPort = Port;
		Connection->m_iFinalPort = FinalPort;
		Connection->m_iConnectionTimeOut = ConnectionTimeOut;
		Connection->b_UseSSL = UseSSL;
		Connection->m_Certificate = CertificateHere;
		Connection->m_CertificatePassword = CertificatePassword;
		Connection->m_iProxyMode = ProxyMode;
		Connection->sSocket.bSslConnection = UseSSL;
		Connection->sSocket.CertificateFile = CertificateHere;
		Connection->sSocket.PrivateKeyPassword = CertificatePassword;
		m_vConnections.push_back((void *)Connection);
		FoundConnection = Connection;
	}

	FoundConnection->IncInuse();

	// We must unlock before trying to connect otherwise we block all other threads
	Unlock();

//	if(!FoundConnection->Connect())
//	{
//		FoundConnection->DecInUse();
//		return NULL;
//	}
	return FoundConnection;
}


void NvHttpConnectionList::FreeConnection(NvHttpConnection* Connection)
{
	Lock();
	if(Connection != NULL)
		Connection->DecInUse();
	Unlock();
}


void NvHttpConnectionList::RemoveConnection(NvHttpConnection* Connection)
{
	int i;
	Lock();
	if(Connection != NULL)
		delete Connection;
	for(i=0; i<(int)m_vConnections.size(); i++)
	{
		NvHttpConnection *ConnectionInList = (NvHttpConnection *)m_vConnections[i];
		if(ConnectionInList == Connection)
			break;
	}
	if(i != m_vConnections.size())
		m_vConnections.erase(m_vConnections.begin()+i);

	Unlock();
}


void NvHttpConnectionList::Update()
{
	Lock();

	int i;
	vector<void *> m_vOldConnections;
	m_vOldConnections = m_vConnections;
	m_vConnections.clear();

	for(i=0; i<(int)m_vOldConnections.size(); i++)
	{
		NvHttpConnection *Connection = (NvHttpConnection *)m_vOldConnections[i];
		if(Connection == NULL)
			continue;
		if((!(Connection->IsInUse())) && (Connection->IsTimeOut()))
		{
			delete Connection;
			continue;
		}
		m_vConnections.push_back((void *)Connection);
	}
	m_vOldConnections.clear();
	Unlock();
}




/////////////////////////////////////////////////////////////////////////////
// NvHttpConnection

NvHttpConnection::NvHttpConnection()
{
	InMessage.sSocket = &sSocket;
	OutMessage.sSocket = &sSocket;
	InUse = 0;
	iTimeOut = NV_CONNECTION_DEFAULT_TIMEOUT;
	tLastUseTime = time(NULL);
	m_sServer = "127.0.0.1";
	m_sFinalAddress = "";
	m_iPort = 1081;
	b_UseSSL = false;
	b_ProxyHttps = false;
	m_iProxyMode = NO_PROXY;
	m_iFinalPort = 1081;
	m_Certificate = "";
	m_CertificatePassword = "";
}

NvHttpConnection::~NvHttpConnection()
{
	Lock();
	sSocket.Close();
	AuthInfo.Clear();
	ProxyAuthInfo.Clear();
	Unlock();
}




void NvHttpConnection::IncInuse()
{
	ProtectInUse.Lock();
	InUse++;
	tLastUseTime = time(NULL);
	ProtectInUse.Unlock();
}

void NvHttpConnection::DecInUse()
{
	ProtectInUse.Lock();
	if(InUse)
	{
		InUse--;
		tLastUseTime = time(NULL);
	}
	ProtectInUse.Unlock();
}

bool NvHttpConnection::IsInUse()
{
	bool RetVal = false;
	ProtectInUse.Lock();
	if(InUse)
		RetVal = true;
	ProtectInUse.Unlock();

	return RetVal;
}


int NvHttpConnection::GetTimeFromLastUse()
{
	time_t tActualTime = time(NULL);
	ProtectInUse.Lock();
	int RetVal = (int)(tActualTime - tLastUseTime);
	ProtectInUse.Unlock();
	return RetVal;
}


bool NvHttpConnection::IsTimeOut()
{
	// A 0 time out means no time out
	// This should not occur because a connection should always have a time out
	if(iTimeOut == 0)
		return false;
	bool RetVal = false;
	time_t tActualTime = time(NULL);
	ProtectInUse.Lock();
	int NumSeconds = (int)(tActualTime - tLastUseTime);
	if(NumSeconds > iTimeOut)
		RetVal = true;
	ProtectInUse.Unlock();
	return RetVal;
}
	

// Send the orders to the server
// return 1 if success, 0 if network error, -1 if the host authentication failed and -2 if proxy authentication failed.
int NvHttpConnection::SendOrder(bool UseDataFile, int DataFileMinSize, const char *WorkDir, string& ErrorInfo)
{
	ErrorInfo = "";

	if(b_ProxyHttps)
		return SendOrderProxyHttps(UseDataFile, DataFileMinSize, WorkDir, ErrorInfo);
	
	bool bProxyAuthRequest = false;
	bool bAuthRequest = false;

	Lock();
#ifdef WIN32
	AuthInfo.FirstStep = true;
	ProxyAuthInfo.FirstStep = true;
#endif

	int NumLoops = 0;
	while(1)
	{
		NumLoops++;
		if(NumLoops > 10)
		{
			// THis loop conunter is to avoid looping in unmanaged cases
			// The connection failed
			Unlock();
			return 0;
		}
		// we clear the input message
		InMessage.Clear();

		// We add the authentication headers if there are some for both proxy and host
		if(!SetAuthHeaders(&OutMessage, true, ErrorInfo))
		{
			// The connection failed
			Unlock();
			return -2;
		}

		if(!SetAuthHeaders(&OutMessage, false, ErrorInfo))
		{
			// The connection failed
			Unlock();
			return -1;
		}

		// We send the output to client
		int Result = OutMessage.Write();
//		int Result = OutMessage.WriteTest();

		if(Result != 1)
		{
	//		if(sSocket.WaitWrite() == 1)
	//		{
				// We are still connected so there is another error
	//			Unlock();
	//			return false;
	//		}
			// We are probably not still connected so we must reconnect
			if(sSocket.Connect(m_sServer.c_str(), m_iPort, m_iConnectionTimeOut) != 1)
			{
				// The connection failed
				Unlock();
				return 0;
			}

			// Let's try again after the reconnection
			Result = OutMessage.Write();
//			Result = OutMessage.WriteTest();
		}
		
		// We retry again after a short sleep
		if(Result != 1)
		{
			SLEEP(1000);
	//		if(sSocket.WaitWrite() == 1)
	//		{
				// We are still connected so there is another error
	//			Unlock();
	//			return false;
	//		}
			// We are probably not still connected so we must reconnect (the connection time out is then 5 seconds)
			if(sSocket.Connect(m_sServer.c_str(), m_iPort, 5) != 1)
			{
				// The connection failed
				Unlock();
				return 0;
			}

			// Let's try again after the reconnection
			Result = OutMessage.Write();
//			Result = OutMessage.WriteTest();
			if(Result != 1)
			{
				Unlock();
				return 0;
			}
		}

		Result = InMessage.Read(0, UseDataFile, DataFileMinSize, WorkDir);	// We wait infinitly
		if(Result < 0)	// error
		{
			// We are maybe not still connected so we must reconnect
			if(sSocket.Connect(m_sServer.c_str(), m_iPort, m_iConnectionTimeOut) != 1)
			{
				// The connection failed
				Unlock();
				return 0;
			}

			// Let's try again the complete message after the reconnection
			Result = OutMessage.Write();
			if(Result != 1)
			{
				Unlock();
				return 0;
			}
			Result = InMessage.Read(0, UseDataFile, DataFileMinSize, NULL);	// We wait infinitly
			if(Result < 0)	// error
			{
				Unlock();
				return 0;
			}
		}

		string sInfo = "";
		if(InMessage.vDatas.size())
			sInfo = InMessage.vDatas[0]->sInfo;
		MakeUpper(sInfo);
		if(sInfo.find("100 CONTINUE") != string::npos)
		{
			InMessage.Clear();
			Result = InMessage.Read(0, UseDataFile, DataFileMinSize, WorkDir);	// We wait infinitly
			if(Result < 0)	// error
			{
				Unlock();
				return 0;
			}
		}

		if(sInfo.find("403 FORBIDDEN") != string::npos)
		{
			// Access error
			Unlock();
			return 0;
		}

		// Check if there is an authentication request from proxy
		if(CheckAuthRequest(&InMessage, true))
		{
			// If we ever checked that, so this is an authentication error

			if((ProxyAuthInfo.m_iMode == HTTP_AUTHENTICATION_BASIC) || (ProxyAuthInfo.m_iMode == HTTP_AUTHENTICATION_DIGEST))
			{
				if(bProxyAuthRequest)
				{
					Unlock();
					return -2;
				}
			}
			// We get the authentication headers for proxy
			if(!GetAuthRequestHeaders(&InMessage, true))
			{
				Unlock();
				return -2;
			}
			
			// And retry;
			bProxyAuthRequest = true;
			continue;
		}
		else
			ProxyAuthInfo.m_iMode = HTTP_AUTHENTICATION_NONE;	// We reset the autehntication mode
		
		// Get the correct answer authentication headers for proxy
		GetAuthHeaders(&InMessage, true);

		// Check if there is an authentication request from host
		if(CheckAuthRequest(&InMessage, false))
		{
			// If we ever checked that, so this is an authentication error
			if((AuthInfo.m_iMode == HTTP_AUTHENTICATION_BASIC) || (AuthInfo.m_iMode == HTTP_AUTHENTICATION_DIGEST))
			{
				if(bAuthRequest)
				{
					Unlock();
					return -1;
				}
			}

			// We get the authentication headers for host
			if(!GetAuthRequestHeaders(&InMessage, false))
			{
				Unlock();
				return -1;
			}
			// And retry;
			bAuthRequest = true;
			continue;
		}
		else
			AuthInfo.m_iMode = HTTP_AUTHENTICATION_NONE;	// We reset the autehntication mode
		
		// Get the correct answer authentication headers for host
		GetAuthHeaders(&InMessage, false);
		break;
	}

	Unlock();
	return 1;
}



// Send the orders to the server when using https via proxy
// In that case, we must tunnel the proxy via an HTTP CONNECT order
int NvHttpConnection::SendOrderProxyHttps(bool UseDataFile, int DataFileMinSize, const char *WorkDir, string& ErrorInfo)
{
	ErrorInfo = "";
	
	bool bProxyAuthRequest = false;
	bool bAuthRequest = false;

	NvHttpProtocol TmpInMessage;
	NvHttpProtocol TmpOutMessage;


	Lock();
#ifdef WIN32
	AuthInfo.FirstStep = true;
	ProxyAuthInfo.FirstStep = true;
#endif

	sSocket.bSslConnection = true;

	bool bInNegotiation = false;

	int NumLoops = 0;
	while(1)
	{
		NumLoops++;
		if(NumLoops > 10)
		{
			// THis loop conunter is to avoid looping in unmanaged cases
			// The connection failed
			Unlock();
			sSocket.Close();
			sSocket.bSslConnection = true;
			return 0;
		}

		// we clear the input message
		InMessage.Clear();

		// We add the authentication headers if there are some for host
		if(!SetAuthHeaders(&OutMessage, false, ErrorInfo))
		{
			// The connection failed
			Unlock();
			sSocket.Close();
			sSocket.bSslConnection = true;
			return -1;
		}

		// We send the output to client

		int Result = OutMessage.Write();;

		if(Result != 1)
		{
			bInNegotiation = true;	// We must negociate with proxy
			sSocket.Close();
			sSocket.bSslConnection = false;

			int NumLoops2 = 0;
			while(bInNegotiation)
			{
				NumLoops2++;
				if(NumLoops2 > 10)
				{
					// THis loop conunter is to avoid looping in unmanaged cases
					// The connection failed
					Unlock();
					sSocket.Close();
					sSocket.bSslConnection = true;
					return 0;
				}
		//		if(sSocket.WaitWrite() == 1)
		//		{
					// We are still connected so there is another error
		//			Unlock();
		//			return false;
		//		}
				// We are probably not still connected so we must reconnect
				if(Result != 1)
				{
					if(sSocket.Connect(m_sServer.c_str(), m_iPort, m_iConnectionTimeOut) != 1)
					{
						// The connection failed
						Unlock();
						sSocket.Close();
						sSocket.bSslConnection = true;
						return 0;
					}
				}
				//sSocket.bSslConnection = false;

				// We must first negociate authentication
				TmpInMessage.Clear();
				TmpOutMessage.Clear();
				TmpInMessage.sSocket = &sSocket;
				TmpOutMessage.sSocket = &sSocket;
				NvData *Data = new NvData;
				TmpOutMessage.vDatas.push_back(Data);

				// We add the authentication headers if there are some for proxy
				if(!SetAuthHeaders(&TmpOutMessage, true, ErrorInfo))
				{
					// The connection failed
					Unlock();
					sSocket.Close();
					sSocket.bSslConnection = true;
					return -2;
				}
				Data->sInfo = "CONNECT ";
				Data->sInfo += OutMessage.GetHeader("Host"); 
				Data->sInfo += " HTTP/1.1"; 
				TmpOutMessage.AddHeader("Host", OutMessage.GetHeader("Host"));
				TmpOutMessage.AddHeader("Content-Length", "0");
				TmpOutMessage.AddHeader("User-Agent", "Nirva");
				TmpOutMessage.AddHeader("Connection", "Keep-Alive");
				TmpOutMessage.AddHeader("Proxy-Connection", "Keep-Alive");

				// Let's try again after the reconnection
				Result = TmpOutMessage.Write();
				if(Result != 1)
				{
					Unlock();
					sSocket.Close();
					sSocket.bSslConnection = true;
					return 0;
				}

				// So we get connected to the proxy
				Result = TmpInMessage.Read();	// We wait infinitly
				if(Result < 0)	// error
				{
					// We are maybe not still connected so we must reconnect
					if(sSocket.Connect(m_sServer.c_str(), m_iPort, m_iConnectionTimeOut) != 1)
					{
						// The connection failed
						Unlock();
						sSocket.Close();
						sSocket.bSslConnection = true;
						return 0;
					}

					// Let's try again the complete after the reconnection
					Result = TmpOutMessage.Write();
					if(Result != 1)
					{
						Unlock();
						sSocket.Close();
						sSocket.bSslConnection = true;
						return 0;
					}
					Result = TmpInMessage.Read();	// We wait infinitly
					if(Result < 0)	// error
					{
						Unlock();
						sSocket.Close();
						sSocket.bSslConnection = true;
						return 0;
					}
					else
					{
						string sInfo = "";
						if(TmpInMessage.vDatas.size())
							sInfo = TmpInMessage.vDatas[0]->sInfo;
						MakeUpper(sInfo);
						if(sInfo.find("100 CONTINUE") != string::npos)
						{
							TmpInMessage.Clear();
							Result = TmpInMessage.Read();	// We wait infinitly
							if(Result < 0)	// error
							{
								Unlock();
								sSocket.Close();
								sSocket.bSslConnection = true;
								return 0;
							}
						}
						if(sInfo.find("403 FORBIDDEN") != string::npos)
						{
							// Access error
							Unlock();
							sSocket.Close();
							sSocket.bSslConnection = true;
							return 0;
						}
					}
				}
				else
				{
					string sInfo = "";
					if(TmpInMessage.vDatas.size())
						sInfo = TmpInMessage.vDatas[0]->sInfo;
					MakeUpper(sInfo);
					if(sInfo.find("100 CONTINUE") != string::npos)
					{
						TmpInMessage.Clear();
						Result = TmpInMessage.Read();	// We wait infinitly
						if(Result < 0)	// error
						{
							Unlock();
							sSocket.Close();
							sSocket.bSslConnection = true;
							return 0;
						}
					}
					if(sInfo.find("403 FORBIDDEN") != string::npos)
					{
						// Access error
						Unlock();
						sSocket.Close();
						sSocket.bSslConnection = true;
						return 0;
					}
				}
			
				// Check if there is an authentication request from proxy
				if(CheckAuthRequest(&TmpInMessage, true))
				{
					// If we ever checked that, so this is an authentication error
					if((ProxyAuthInfo.m_iMode == HTTP_AUTHENTICATION_BASIC) || (ProxyAuthInfo.m_iMode == HTTP_AUTHENTICATION_DIGEST))
					{
						if(bProxyAuthRequest)
						{
							Unlock();
							sSocket.Close();
							sSocket.bSslConnection = true;
							return -2;
						}
					}

					// We get the authentication headers for proxy
					if(!GetAuthRequestHeaders(&TmpInMessage, true))
					{
						Unlock();
						sSocket.Close();
						sSocket.bSslConnection = true;
						return -2;
					}
					// And retry;
					bProxyAuthRequest = true;
				}
				else
					ProxyAuthInfo.m_iMode = HTTP_AUTHENTICATION_NONE;	// We reset the autehntication mode
				
				// Get the correct answer authentication headers for proxy
				GetAuthHeaders(&TmpInMessage, true);


				// Check if we got a 200 Connection established answer
				string sInfo = "";
				if(TmpInMessage.vDatas.size())
					sInfo = TmpInMessage.vDatas[0]->sInfo;
				MakeUpper(sInfo);
				TmpInMessage.Clear();
				TmpOutMessage.Clear();
				if(sInfo.find("200") != string::npos)
				{
					// We are connected and identified to the proxy
					bInNegotiation = false;
					// The proxy tunnel has been established
					// So we can now work in SSL
					sSocket.bSslConnection = true;
					if(!sSocket.SslInit())
					{
						Unlock();
						sSocket.Close();
						return 0;
					}
				}
				// Here we are still in negociation
			}

			Result = OutMessage.Write();
			if(Result != 1)
			{
				Unlock();
				sSocket.Close();
				return 0;
			}
		}

		Result = InMessage.Read(0, UseDataFile, DataFileMinSize, WorkDir);	// We wait infinitly
		if(Result < 0)	// error
		{
			Unlock();
			sSocket.Close();
			return 0;
		}
		else
		{
			string sInfo = "";
			if(InMessage.vDatas.size())
				sInfo = InMessage.vDatas[0]->sInfo;
			MakeUpper(sInfo);
			if(sInfo.find("100 CONTINUE") != string::npos)
			{
				InMessage.Clear();
				Result = InMessage.Read(0, UseDataFile, DataFileMinSize, WorkDir);	// We wait infinitly
				if(Result < 0)	// error
				{
					Unlock();
					sSocket.Close();
					return 0;
				}
			}
			if(sInfo.find("403 FORBIDDEN") != string::npos)
			{
				// Access error
				Unlock();
				sSocket.Close();
				return 0;
			}
		}

		// Check if there is an authentication request from host
		if(CheckAuthRequest(&InMessage, false))
		{
			// If we ever checked that, so this is an authentication error
			if((AuthInfo.m_iMode == HTTP_AUTHENTICATION_BASIC) || (AuthInfo.m_iMode == HTTP_AUTHENTICATION_DIGEST))
			{
				if(bAuthRequest)
				{
					Unlock();
					sSocket.Close();
					return -1;
				}
			}

			// We get the authentication headers for host
			if(!GetAuthRequestHeaders(&InMessage, false))
			{
				Unlock();
				sSocket.Close();
				return -1;
			}
			
			// And retry;
			bAuthRequest = true;
			continue;
		}
		else
			AuthInfo.m_iMode = HTTP_AUTHENTICATION_NONE;	// We reset the autehntication mode
		
		// Get the correct answer authentication headers for host
		GetAuthHeaders(&InMessage, false);
		break;
	}

	Unlock();
	return true;
}
	

// Connects the server if not connected
bool NvHttpConnection::Connect()
{

	Lock();

	if(sSocket.WaitWrite() != 1)
	{
		// We are not still connected so we must reconnect
		if(sSocket.Connect(m_sServer.c_str(), m_iPort, m_iConnectionTimeOut) != 1)
		{
			Unlock();
			return false;
		}
	}
	Unlock();
	
	return true;
}


// Check if the server or proxy requests an authentication
bool NvHttpConnection::CheckAuthRequest(NvHttpProtocol* Message, bool Proxy)
{
	string CommandHere = Message->sCommand;
	MakeUpper(CommandHere);
	CleanString(CommandHere);
	string StringToSearch = "401";
	if(Proxy)
		StringToSearch = "407";

	if(CommandHere.find(StringToSearch) == 0)
		return true;

	return false;
}

// Get the headers information from authentication request response
bool NvHttpConnection::GetAuthRequestHeaders(NvHttpProtocol* Message, bool Proxy)
{
	NvAuthInfo *AuthInfoHere = &AuthInfo;
	string HttpHeaderName = "WWW-AUTHENTICATE";
	if(Proxy)
	{
		HttpHeaderName = "PROXY-AUTHENTICATE";
		AuthInfoHere = &ProxyAuthInfo;
	}
	
	string HttpHeader = Message->GetHeader(HttpHeaderName);

	// eliminate first and last spaces
	while(HttpHeader.size() && HttpHeader[0] == ' ')
		HttpHeader.erase(0, 1);
	while(HttpHeader.size() && HttpHeader[HttpHeader.size()-1] == ' ')
		HttpHeader.erase(HttpHeader.size()-1, 1);

	string::size_type index = HttpHeader.find(' ');
	string Method = HttpHeader;
	if(index != string::npos)
	{
		Method = HttpHeader.substr(0, index);
		HttpHeader = HttpHeader.substr(index+1);
	}
	else
		HttpHeader = "";
	CleanString(Method);
	MakeUpper(Method);
	AuthInfoHere->m_iMode = HTTP_AUTHENTICATION_BASIC;
	if(Method == "DIGEST")
		AuthInfoHere->m_iMode = HTTP_AUTHENTICATION_DIGEST;
#ifdef WIN32
	if(Method == "NTLM")
		AuthInfoHere->m_iMode = HTTP_AUTHENTICATION_NTLM;
	if(Method == "NEGOTIATE")
		AuthInfoHere->m_iMode = HTTP_AUTHENTICATION_NEGOTIATE;
	if(Method == "KERBEROS")
		AuthInfoHere->m_iMode = HTTP_AUTHENTICATION_KERBEROS;
#endif
	if((AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_NTLM) || (AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_NEGOTIATE) || (AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_KERBEROS))
		if(!AuthInfoHere->SsoInfo.FirstStep && !HttpHeader.length())
			return false; // if there is no data and we have started the negociation, this is an error 

	// Parse the header data now
	string HttpHeaderOri = HttpHeader;
	map<string, string> Headers;
	while(HttpHeader.length())
	{
		string KeyValue = "";
		index = HttpHeader.find(',');
		if(index == string::npos)
		{
			KeyValue = HttpHeader;
			HttpHeader = "";
		}
		else
		{
			KeyValue = HttpHeader.substr(0, index);
			HttpHeader = HttpHeader.substr(index+1);
		}
		string KeyHere = "";
		string ValueHere = "";
		index = KeyValue.find('=');
		if(index == string::npos)
		{
			KeyHere = KeyValue;
			ValueHere = "";
		}
		else
		{
			KeyHere = KeyValue.substr(0, index);
			ValueHere = KeyValue.substr(index+1);
		}
		if(KeyHere.length())
		{
			CleanString(KeyHere);
			MakeUpper(KeyHere);
			// eliminate first and last spaces
			while(ValueHere.size() && ValueHere[0] == ' ')
				ValueHere.erase(0, 1);
			while(ValueHere.size() && ValueHere[ValueHere.size()-1] == ' ')
				ValueHere.erase(ValueHere.size()-1, 1);
			// eliminate first and last double quotes
			if(ValueHere.size() && ValueHere[0] == '\"')
				ValueHere.erase(0, 1);
			if(ValueHere.size() && ValueHere[ValueHere.size()-1] == '\"')
				ValueHere.erase(ValueHere.size()-1, 1);
			Headers[KeyHere] = ValueHere;
		}
	}
	
	if(AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_BASIC)
	{
		// basic
		AuthInfoHere->m_sRealm = "";
		if(Headers.find("REALM") != Headers.end())
			AuthInfoHere->m_sRealm = Headers["REALM"];
		AuthInfoHere->m_sDomain = "";
		AuthInfoHere->m_sNonce = "";
		AuthInfoHere->m_sOpaque = "";
		AuthInfoHere->m_bStale = false;
		AuthInfoHere->m_sAlgorithm = "MD5";
		AuthInfoHere->m_sQop = "";
		AuthInfoHere->m_iNc = 1;
		AuthInfoHere->m_sCnonce = "";
	}
#ifdef WIN32
	else if((AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_NTLM) || (AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_NEGOTIATE) || (AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_KERBEROS))
	{
		// ntlm, negotiate, kerberos
		// The data is in HttpHeaderOri
		if(HttpHeaderOri.length())
		{
			// prepare to get the server's response
			if(AuthInfoHere->SsoInfo.ib.pvBuffer != NULL)
			{
				free(AuthInfoHere->SsoInfo.ib.pvBuffer);
				AuthInfoHere->SsoInfo.ib.pvBuffer = NULL;
			}
			AuthInfoHere->SsoInfo.ibd.ulVersion = SECBUFFER_VERSION;
			AuthInfoHere->SsoInfo.ibd.cBuffers = 1;
			AuthInfoHere->SsoInfo.ibd.pBuffers = &(AuthInfoHere->SsoInfo.ib); // just one buffer
			AuthInfoHere->SsoInfo.ib.BufferType = SECBUFFER_TOKEN; // preping a token here

			char *Buffer = NULL;
			int DataSize = (int)(HttpHeaderOri.length());
			MemBase64Decode(HttpHeaderOri.c_str(), (char **)(&(AuthInfoHere->SsoInfo.ib.pvBuffer)), &DataSize);
			AuthInfoHere->SsoInfo.ib.cbBuffer = (unsigned long)(DataSize);
			AuthInfoHere->SsoInfo.haveInbuffer = true;
			string Hash = Message->GetHeader("NV_AUTHENTICATE_DATA_HASH");
			if(Hash.length())
			{
				string CalculatedHash = MD5_string(HttpHeaderOri);
				if(CalculatedHash != Hash)
					return false;
			}
		}
		else
		{
			// We have no data after NTLM string from the server so this is probably the first step
		}
	}
#endif
	else
	{
		// digest
		AuthInfoHere->m_sRealm = "";
		if(Headers.find("REALM") != Headers.end())
			AuthInfoHere->m_sRealm = Headers["REALM"];
		AuthInfoHere->m_sDomain = "";
		if(Headers.find("DOMAIN") != Headers.end())
			AuthInfoHere->m_sDomain = Headers["DOMAIN"];
		AuthInfoHere->m_sNonce = "";
		if(Headers.find("NONCE") != Headers.end())
			AuthInfoHere->m_sNonce = Headers["NONCE"];
		AuthInfoHere->m_sOpaque = "";
		if(Headers.find("OPAQUE") != Headers.end())
			AuthInfoHere->m_sOpaque = Headers["OPAQUE"];
		AuthInfoHere->m_bStale = false;
		if(Headers.find("STALE") != Headers.end())
		{
			string sStale = Headers["STALE"];
			CleanString(sStale);
			MakeUpper(sStale);
			if(sStale == "TRUE")
			{
				AuthInfoHere->m_bStale = true;
				AuthInfoHere->m_iNc = 1;
			}
		}
		AuthInfoHere->m_sAlgorithm = "MD5";
		if(Headers.find("ALGORITHM") != Headers.end())
			AuthInfoHere->m_sAlgorithm = Headers["ALGORITHM"];
		CleanString(AuthInfoHere->m_sAlgorithm);
		MakeUpper(AuthInfoHere->m_sAlgorithm);
		AuthInfoHere->m_sQop = "";
		if(Headers.find("QOP") != Headers.end())
			AuthInfoHere->m_sQop = Headers["QOP"];
		CleanString(AuthInfoHere->m_sQop);
		if(AuthInfoHere->m_sQop.length())
			AuthInfoHere->m_sQop = "auth";
		AuthInfoHere->m_sCnonce = "";
	}

	return true;
}

// Get the headers information from correct server response
void NvHttpConnection::GetAuthHeaders(NvHttpProtocol* Message, bool Proxy)
{
	NvAuthInfo *AuthInfoHere = &AuthInfo;
	if(Proxy)
	{
		AuthInfoHere = &ProxyAuthInfo;
	}

	if(AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_NONE)
		return;
	// Nothing to do here for Basic authentication
	if(AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_BASIC)
		return;
	
	if(AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_DIGEST)
	{
		string HttpHeaderName = "AUTHENTICATION-INFO";
		if(Proxy)
		{
			HttpHeaderName = "PROXY-AUTHENTICATION-INFO";
			AuthInfoHere = &ProxyAuthInfo;
		}
		
		string HttpHeader = Message->GetHeader(HttpHeaderName);
		if(!HttpHeader.length())
			return;

		// eliminate first and last spaces
		while(HttpHeader.size() && HttpHeader[0] == ' ')
			HttpHeader.erase(0, 1);
		while(HttpHeader.size() && HttpHeader[HttpHeader.size()-1] == ' ')
			HttpHeader.erase(HttpHeader.size()-1, 1);

		// Parse the header data now
		map<string, string> Headers;
		while(HttpHeader.length())
		{
			string KeyValue = "";
			string::size_type index = HttpHeader.find(',');
			if(index == string::npos)
			{
				KeyValue = HttpHeader;
				HttpHeader = "";
			}
			else
			{
				KeyValue = HttpHeader.substr(0, index);
				HttpHeader = HttpHeader.substr(index+1);
			}
			string KeyHere = "";
			string ValueHere = "";
			index = KeyValue.find('=');
			if(index == string::npos)
			{
				KeyHere = KeyValue;
				ValueHere = "";
			}
			else
			{
				KeyHere = KeyValue.substr(0, index);
				ValueHere = KeyValue.substr(index+1);
			}
			if(KeyHere.length())
			{
				CleanString(KeyHere);
				MakeUpper(KeyHere);
				// eliminate first and last spaces
				while(ValueHere.size() && ValueHere[0] == ' ')
					ValueHere.erase(0, 1);
				while(ValueHere.size() && ValueHere[ValueHere.size()-1] == ' ')
					ValueHere.erase(ValueHere.size()-1, 1);
				// eliminate first and last double quotes
				if(ValueHere.size() && ValueHere[0] == '\"')
					ValueHere.erase(0, 1);
				if(ValueHere.size() && ValueHere[ValueHere.size()-1] == '\"')
					ValueHere.erase(ValueHere.size()-1, 1);
				Headers[KeyHere] = ValueHere;
			}
		}
		
		if(Headers.find("NEXTNONCE") != Headers.end())
		{
			AuthInfoHere->m_sNonce = Headers["NEXTNONCE"];
			AuthInfoHere->m_iNc = 1;
		}
	}
	else if((AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_NTLM) || (AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_NEGOTIATE) || (AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_KERBEROS))
	{
		// nothing to do here since this has been done in GetAuthRequestHeaders
	}
}


// Set the response authentication headers
bool NvHttpConnection::SetAuthHeaders(NvHttpProtocol* Message, bool Proxy, string& ErrorInfo)
{
	ErrorInfo = "";
	NvAuthInfo *AuthInfoHere = &AuthInfo;
	if(Proxy)
	{
		AuthInfoHere = &ProxyAuthInfo;
	}

	Message->AddHeader("User-Agent", "Nirva");
	Message->AddHeader("Connection", "Keep-Alive");
	Message->AddHeader("Proxy-Connection", "Keep-Alive");

	if(AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_NONE)
		return true;

	if(AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_DIGEST)
	{
		if(!AuthInfoHere->m_sUser.length())
			return true;
		// Process for digest authentication
		if(!AuthInfoHere->m_sNonce.length())
			return true;
		if(!AuthInfoHere->m_iNc)
			AuthInfoHere->m_iNc = 1;

		// Create a new cnonce if required
		if(!AuthInfoHere->m_sCnonce.length())
		{
			time_t actualtime = time(NULL);
			char Buf[20];
			sprintf(Buf, "%ld", actualtime);
			AuthInfoHere->m_sCnonce = Buf;
			AuthInfoHere->m_sCnonce = MD5_string(AuthInfoHere->m_sCnonce);
			AuthInfoHere->m_sCnonce = AuthInfoHere->m_sCnonce.substr(0,8);
		}
		
		// if the algorithm is "MD5" or unspecified (which then defaults to MD5):
		// A1 = unq(username-value) ":" unq(realm-value) ":" passwd
		// if the algorithm is "MD5-sess" then:
		// A1 = H( unq(username-value) ":" unq(realm-value) ":" passwd )
		//		":" unq(nonce-value) ":" unq(cnonce-value)

		string hA1;
		hA1 = AuthInfoHere->m_sUser;
		hA1 += ":";
		hA1 += AuthInfoHere->m_sRealm;
		hA1 += ":";
		hA1 += AuthInfoHere->m_sPassword;
		hA1 = MD5_string(hA1);
		if(AuthInfoHere->m_sAlgorithm == "MD5-SESS")
		{
			hA1 += ":";
			hA1 += AuthInfoHere->m_sNonce;
			hA1 += ":";
			hA1 += AuthInfoHere->m_sCnonce;
			hA1 = MD5_string(hA1);
		}

		// If the "qop" directive's value is "auth" or is unspecified, then A2 is:
		// A2       = Method ":" digest-uri-value
		// If the "qop" value is "auth-int", then A2 is:
		// A2       = Method ":" digest-uri-value ":" H(entity-body)
		// (The "Method" value is the HTTP request method as specified in section
		// 5.1.1 of RFC 2616)
		// For now only the auth is accepted by Nirva

		// First get the method and URI from request
		string Method = Message->vDatas[0]->sInfo;
		string Uri = "";
		string::size_type index = Method.find(' ');
		if(index != string::npos)
		{
			Uri = Method.substr(index+1);
			Method = Method.substr(0, index);
		}
		index = Uri.find(' ');
		if(index != string::npos)
			Uri = Uri.substr(0, index);

		string hA2 = Method;
		hA2 += ":";
		hA2 += Uri;
		hA2 = MD5_string(hA2);

		string Response = hA1;
		Response += ":";
		Response += AuthInfoHere->m_sNonce;
		Response += ":";
		if(AuthInfoHere->m_sQop.length())
		{
			char Buf[20];
			sprintf(Buf, "%08x", AuthInfoHere->m_iNc);
			Response += Buf;
			Response += ":";
			Response += AuthInfoHere->m_sCnonce;
			Response += ":";
			Response += AuthInfoHere->m_sQop;
		}
		Response += ":";
		Response += hA2;
		Response = MD5_string(Response);

		string Authentication = "username=\"";
		Authentication += AuthInfoHere->m_sUser;
		Authentication += "\" ,realm=\"";
		Authentication += AuthInfoHere->m_sRealm;
		Authentication += "\" ,nonce=\"";
		Authentication += AuthInfoHere->m_sNonce;
		Authentication += "\" ,uri=\"";
		Authentication += Uri;
		if(AuthInfoHere->m_sQop.length())
		{
			Authentication += "\" ,cnonce=\"";
			Authentication += AuthInfoHere->m_sCnonce;
			char Buf[20];
			sprintf(Buf, "%08x", AuthInfoHere->m_iNc);
			Authentication += "\" ,nc=";
			Authentication += Buf;
			Authentication += " ,qop=\"";
			Authentication += AuthInfoHere->m_sQop;
		}
		Authentication += "\" ,response=\"";
		Authentication += Response;
		Authentication += "\"";
	
		if(AuthInfoHere->m_sQop.length())
			AuthInfoHere->m_iNc++;

		if(AuthInfoHere->m_sOpaque.length())
		{
			Authentication += " ,opaque=\"";
			Authentication += AuthInfoHere->m_sOpaque;
			Authentication += "\"";
		}

		if(AuthInfoHere->m_sAlgorithm.length())
		{
			Authentication += " ,algorithm=\"";
			Authentication += AuthInfoHere->m_sAlgorithm;
			Authentication += "\"";
		}
	
		string AuthenticationHeader = "Digest ";
		AuthenticationHeader += Authentication;

		if(Proxy)
			Message->AddHeader("Proxy-Authorization", AuthenticationHeader);
		else
			Message->AddHeader("Authorization", AuthenticationHeader);
	}
	if(AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_BASIC)
	{
		if(!AuthInfoHere->m_sUser.length())
			return true;

		// Process for basic authentication
		string Authentication = AuthInfoHere->m_sUser;
		Authentication += ":";
		Authentication += AuthInfoHere->m_sPassword;
		char *Buffer = NULL;
		int DataSize = (int)(Authentication.length());
		MemBase64Encode(Authentication.c_str(), &Buffer, &DataSize, false);
		Authentication = Buffer;
		free(Buffer);
		CleanString(Authentication);
		string AuthenticationHeader = "Basic ";
		AuthenticationHeader += Authentication;

		if(Proxy)
			Message->AddHeader("Proxy-Authorization", AuthenticationHeader);
		else
			Message->AddHeader("Authorization", AuthenticationHeader);
	}
	string AuthMethod = "";
	if(AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_NTLM)
		AuthMethod = "NTLM";
	if(AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_NEGOTIATE)
		AuthMethod = "Negotiate";
	if(AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_KERBEROS)
		AuthMethod = "Kerberos";
	if((AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_NTLM) || (AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_NEGOTIATE) || (AuthInfoHere->m_iMode == HTTP_AUTHENTICATION_KERBEROS))
	{
#ifdef WIN32
		SECURITY_STATUS err;
		if(AuthInfoHere->SsoInfo.FirstStep)
		{
			// we go one or 2 times max in this loop
			// 1 if the original method is not NTLM, 2 otherwise (we retry with NTLM)
			while(1)
			{
				AuthInfoHere->SsoInfo.haveInbuffer = FALSE;
				AuthInfoHere->SsoInfo.haveContext = FALSE;
				if(AuthInfoHere->SsoInfo.ib.pvBuffer != NULL)
				{
					free(AuthInfoHere->SsoInfo.ib.pvBuffer);
					AuthInfoHere->SsoInfo.ib.pvBuffer = NULL;
				}
				if(AuthInfoHere->SsoInfo.ob.pvBuffer != NULL)
				{
					free(AuthInfoHere->SsoInfo.ob.pvBuffer);
					AuthInfoHere->SsoInfo.ob.pvBuffer = NULL;
				}
				if(AuthInfoHere->SsoInfo.secPackInfo != NULL)
					FreeContextBuffer(AuthInfoHere->SsoInfo.secPackInfo);
				AuthInfoHere->SsoInfo.secPackInfo = NULL;
				if(AuthInfoHere->SsoInfo.bValidcliCtx)
					DeleteSecurityContext(&(AuthInfoHere->SsoInfo.cliCtx));
				if(AuthInfoHere->SsoInfo.bValidhcredClient)
					FreeCredentialHandle(&(AuthInfoHere->SsoInfo.hcredClient));
				AuthInfoHere->SsoInfo.bValidhcredClient = false;
				AuthInfoHere->SsoInfo.bValidcliCtx = false;
				// Check if the user contains the domain
				AuthInfoHere->m_sDomain = "";
				if(AuthInfoHere->m_sUser.length())
				{
					string::size_type index = AuthInfoHere->m_sUser.find('/');
					if(index != string::npos)
					{
						// Yes we have a domain in user name
						AuthInfoHere->m_sDomain = AuthInfoHere->m_sUser.substr(0, index);
						AuthInfoHere->m_sUser = AuthInfoHere->m_sUser.substr(index+1);
					}
				}
				if(!AuthInfoHere->m_sDomain.length())
				{
					//if there is no domain, we use the current domain
					char Buff[1024];
					DWORD dwSize  = 1024;
					GetComputerNameEx((COMPUTER_NAME_FORMAT)ComputerNameDnsDomain, Buff,&dwSize);
					AuthInfoHere->m_sDomain = Buff;
				}

				// if there is still no domain, we are maybe not connected to a domain so we try with NTLM if the method was not NTLM
				if(!AuthInfoHere->m_sDomain.length())
				{
					if(AuthInfoHere->m_iMode != HTTP_AUTHENTICATION_NTLM)
					{
						AuthInfoHere->m_iMode = HTTP_AUTHENTICATION_NTLM;
						AuthMethod = "NTLM";
						continue; // retry with NTLM
					}
				}
				QuerySecurityPackageInfo((SEC_CHAR *)(AuthMethod.c_str()), &(AuthInfoHere->SsoInfo.secPackInfo));
				break;
			}
			if(AuthInfoHere->m_sUser.length())
			{
				// Explicit user
				AuthInfoHere->SsoInfo.nameAndPwd.Domain = (byte *)(AuthInfoHere->m_sDomain.c_str());
				AuthInfoHere->SsoInfo.nameAndPwd.DomainLength = (unsigned long)(AuthInfoHere->m_sDomain.length());
				AuthInfoHere->SsoInfo.nameAndPwd.User = (byte *)(AuthInfoHere->m_sUser.c_str());
				AuthInfoHere->SsoInfo.nameAndPwd.UserLength = (unsigned long)(AuthInfoHere->m_sUser.length());
				AuthInfoHere->SsoInfo.nameAndPwd.Password = (byte *)(AuthInfoHere->m_sPassword.c_str());
				AuthInfoHere->SsoInfo.nameAndPwd.PasswordLength = (unsigned long)(AuthInfoHere->m_sPassword.length());
				AuthInfoHere->SsoInfo.nameAndPwd.Flags = SEC_WINNT_AUTH_IDENTITY_ANSI;
				err = AcquireCredentialsHandle(NULL, (SEC_CHAR *)(AuthMethod.c_str()), SECPKG_CRED_OUTBOUND, NULL, &(AuthInfoHere->SsoInfo.nameAndPwd), NULL, NULL, &(AuthInfoHere->SsoInfo.hcredClient), &(AuthInfoHere->SsoInfo.expiryClient));
			}
			else
			{
				// Current user
				err = AcquireCredentialsHandle(NULL, (SEC_CHAR *)(AuthMethod.c_str()), SECPKG_CRED_OUTBOUND, NULL, NULL, NULL, NULL, &(AuthInfoHere->SsoInfo.hcredClient), &(AuthInfoHere->SsoInfo.expiryClient));
			}
			if(err != SEC_E_OK)
			{
				ErrorInfo = "Cannot get Credentials for current user";
				return false;
			}
			AuthInfoHere->SsoInfo.bValidhcredClient = true;
			AuthInfoHere->SsoInfo.FirstStep = false;
		}
		AuthInfoHere->SsoInfo.obd.ulVersion = SECBUFFER_VERSION;
		AuthInfoHere->SsoInfo.obd.cBuffers = 1;
		AuthInfoHere->SsoInfo.obd.pBuffers = &(AuthInfoHere->SsoInfo.ob); // just one buffer
		AuthInfoHere->SsoInfo.ob.BufferType = SECBUFFER_TOKEN; // preping a token here
		AuthInfoHere->SsoInfo.ob.cbBuffer = AuthInfoHere->SsoInfo.secPackInfo->cbMaxToken +1;
		AuthInfoHere->SsoInfo.ob.pvBuffer = malloc(AuthInfoHere->SsoInfo.ob.cbBuffer);

		// Set the principal name
		// For a proxy, the principal is set to "nirva/proxy"
		// For Nirva, the principal is not set by default (it must be set by the client if Kerberos or Kerberos via Negociate is used)
		// In this case, this must be an existing principal defined on the Active Directory server via setspn
		string Principal = "nirva/proxy";
		if(!Proxy)
			Principal = AuthInfoHere->SsoInfo.m_sPrincipal;
		CleanString(Principal);

		DWORD ctxAttr;
		TimeStamp useBefore;

		err = InitializeSecurityContext(&(AuthInfoHere->SsoInfo.hcredClient),
									AuthInfoHere->SsoInfo.haveContext ? &(AuthInfoHere->SsoInfo.cliCtx): NULL,
									(SEC_CHAR *)(Principal.c_str()),
									ISC_REQ_MUTUAL_AUTH | ISC_REQ_REPLAY_DETECT | ISC_REQ_SEQUENCE_DETECT | ISC_REQ_CONFIDENTIALITY | ISC_REQ_DELEGATE,
									NULL,
									SECURITY_NATIVE_DREP,
									AuthInfoHere->SsoInfo.haveInbuffer ? &(AuthInfoHere->SsoInfo.ibd): NULL,
									0,
									&(AuthInfoHere->SsoInfo.cliCtx),
									&(AuthInfoHere->SsoInfo.obd),
									&ctxAttr,
									&useBefore);
/*		char szErr[256];
		if (FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 0, err, 0, szErr, sizeof szErr / sizeof *szErr, 0))
			printf("InitializeSecurityContext failed: %s", szErr);
		if(AuthInfoHere->SsoInfo.ib.pvBuffer != NULL)
		{
			free(AuthInfoHere->SsoInfo.ib.pvBuffer);
			AuthInfoHere->SsoInfo.ib.pvBuffer = NULL;
		}
*/
		if (err == SEC_I_COMPLETE_AND_CONTINUE || err == SEC_I_COMPLETE_NEEDED)
		{
			CompleteAuthToken(&(AuthInfoHere->SsoInfo.cliCtx), &(AuthInfoHere->SsoInfo.obd));
			if (err == SEC_I_COMPLETE_NEEDED)
				err = SEC_E_OK;
			else if (err == SEC_I_COMPLETE_AND_CONTINUE)
				err = SEC_I_CONTINUE_NEEDED;
		}

		if ((err != SEC_E_OK) && (err != SEC_I_CONTINUE_NEEDED))
		{
			ErrorInfo = "Unknown";
			switch(err)
			{
				case SEC_E_INSUFFICIENT_MEMORY:
					ErrorInfo = "SEC_E_INSUFFICIENT_MEMORY";
					break;
				case SEC_E_INTERNAL_ERROR:
					ErrorInfo = "SEC_E_INTERNAL_ERROR";
					break;
				case SEC_E_INVALID_HANDLE:
					ErrorInfo = "SEC_E_INVALID_HANDLE";
					break;
				case SEC_E_INVALID_TOKEN:
					ErrorInfo = "SEC_E_INVALID_TOKEN";
					break;
				case SEC_E_LOGON_DENIED:
					ErrorInfo = "SEC_E_LOGON_DENIED";
					break;
				case SEC_E_NO_AUTHENTICATING_AUTHORITY:
					ErrorInfo = "SEC_E_NO_AUTHENTICATING_AUTHORITY";
					break;
				case SEC_E_NO_CREDENTIALS:
					ErrorInfo = "SEC_E_NO_CREDENTIALS";
					break;
				case SEC_E_TARGET_UNKNOWN:
					ErrorInfo = "SEC_E_TARGET_UNKNOWN";
					break;
				case SEC_E_UNSUPPORTED_FUNCTION:
					ErrorInfo = "SEC_E_UNSUPPORTED_FUNCTION";
					break;
				case SEC_E_WRONG_PRINCIPAL:
					ErrorInfo = "SEC_E_WRONG_PRINCIPAL";
					break;
			}
//printf("Error = %s\n", ErrorMess.c_str());

			return false;
		}
		AuthInfoHere->SsoInfo.bValidcliCtx = true;
		string AuthenticationHeader = AuthMethod;
		AuthenticationHeader += " ";
		char *Buffer = NULL;
		int DataSize = (int)(AuthInfoHere->SsoInfo.ob.cbBuffer);
		MemBase64Encode((const char *)(AuthInfoHere->SsoInfo.ob.pvBuffer), &Buffer, &DataSize, false);
		Buffer[DataSize] = '\0';
		string Hash = MD5_string(Buffer);
		if(!Proxy)
			Message->AddHeader("Nv_authenticate_data_hash", Hash);
		AuthenticationHeader += Buffer;
		free(Buffer);

		if(Proxy)
			Message->AddHeader("Proxy-Authorization", AuthenticationHeader);
		else
			Message->AddHeader("Authorization", AuthenticationHeader);

		if(AuthInfoHere->SsoInfo.ob.pvBuffer != NULL)
		{
			free(AuthInfoHere->SsoInfo.ob.pvBuffer);
			AuthInfoHere->SsoInfo.ob.pvBuffer = NULL;
		}

		// by now we have a client context
		AuthInfoHere->SsoInfo.haveContext = true;

#endif
	}
	return true;
}



/////////////////////////////////////////////////////////////////////////////
// NvSsoInfo

NvSsoInfo::NvSsoInfo()
{
#ifdef WIN32
	haveInbuffer = false;
	haveContext = false;
	FirstStep = true;
	secPackInfo = NULL;
	ib.pvBuffer = NULL;
	ob.pvBuffer = NULL;
	bValidhcredClient = false;
	bValidcliCtx = false;
#endif
}

NvSsoInfo::~NvSsoInfo()
{
	Clear();
}

void NvSsoInfo::Clear()
{
#ifdef WIN32
	haveInbuffer = false;
	haveContext = false;
	FirstStep = true;
	if(secPackInfo != NULL)
		FreeContextBuffer(secPackInfo);
	if(bValidcliCtx)
		DeleteSecurityContext(&cliCtx);
	if(bValidhcredClient)
		FreeCredentialHandle(&hcredClient);
	bValidhcredClient = false;
	bValidcliCtx = false;
	secPackInfo = NULL;
	if(ib.pvBuffer != NULL)
	{
		free(ib.pvBuffer);
		ib.pvBuffer = NULL;
	}
	if(ob.pvBuffer != NULL)
	{
		free(ob.pvBuffer);
		ob.pvBuffer = NULL;
	}
#endif
}



/////////////////////////////////////////////////////////////////////////////
// NvAuthInfo

NvAuthInfo::NvAuthInfo()
{
	m_iMode = HTTP_AUTHENTICATION_NONE;
	m_sUser = "";
	m_sPassword = "";
	m_sRealm = "";
	m_sDomain = "";
	m_sNonce = "";
	m_sOpaque = "";
	m_bStale = false;
	m_sAlgorithm = "MD5";
	m_sQop = "";
	m_iNc = 1;
	m_sCnonce = "";
}

NvAuthInfo::~NvAuthInfo()
{
	Clear();
}

void NvAuthInfo::Clear()
{
	m_iMode = HTTP_AUTHENTICATION_NONE;
	m_sUser = "";
	m_sPassword = "";
	m_sRealm = "";
	m_sDomain = "";
	m_sNonce = "";
	m_sOpaque = "";
	m_bStale = false;
	m_sAlgorithm = "MD5";
	m_sQop = "";
	m_iNc = 1;
	m_sCnonce = "";
	SsoInfo.Clear();
}


