// ***************************************************************************************
// File: global.h
// Description: functions used in ejava.cpp and service.cpp
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifndef _GLOBAL_H
#define _GLOBAL_H

#include "jni.h"
#include "nvsext.h"
#include <string>
#include <vector>
#include <map>



#ifdef WIN32
#define NVTHREADID DWORD
#else
#define NVTHREADID pthread_t
#endif

#define RESULT_BUFFER_LENGTH	1024
#ifdef WIN32
#define PATH_SEPARATOR ";"
#else
#define PATH_SEPARATOR ":"
#endif


int ParseMethodName(const char* MethodNameToParse, std::string& DirName, std::string& ClassName, std::string& MethodName);
bool MyAttachCurrentThread(void **JavaEnv);
void MyDetachCurrentThread();


/////////////////////////////////////////////////////////////////////////////
// NvJavaInt class
//
// This class encapsulates the thread context for the method
//
class NvJavaInt
{
public:
	std::vector<void *> m_mCommands;	// command stack (used for recursivity)
	void* m_pCommand;					// actual command
	char *ResultBuffer;							// command result buffer
	int ResultBufferSize;						// command result buffer size
	std::string LastResult;							// last string result
	int InUse;									// Used for thread attachement
	
public:
	NvJavaInt();
	virtual ~NvJavaInt();

	int SetBufferSize(int DataSize);			// eventually allocate or reallocate the result buffer
};

/////////////////////////////////////////////////////////////////////////////
// NvJavaThreadList class
//
// This class manages the correspondance between the thread handle and the NvJavaInt
// class that is the interface to Nirva
//
class NvJavaThreadList : public NvObject
{
public:
	std::map<NVTHREADID, NvJavaInt*> m_mList;	// list of threads using java
	
public:
	NvJavaThreadList();
	virtual ~NvJavaThreadList();

	// Remove all threads from the list
	void RemoveAll();				
	
	// Eventually attach the current thread and retuns a NvJavaInt class that owns the thread context
	NvJavaInt* GetThreadNvIntPtr(void* pCommand = NULL);
	
	// Eventually detach the current thread and remove the thread context
	void FreeThreadNvIntPtr(bool force = false);
};



/////////////////////////////////////////////////////////////////////////////
// NvJavaEnv class
//
// This class encapsulates the java environment
//
class NvJavaEnv
{
public:
	jobject MyLoader;				// Java class loader
	jobject ClassToRun;				// Pointer to the class object
	jmethodID ms;					// Current method
	jmethodID msLoaderFindClass;	// Find class method of the loader
	jmethodID msRender;				// Render method
	std::string CurrentMethod;	// Current method name
	bool bStaticMethod;			// Set to true if the method is static
	bool bOkOne;				// Set to true if method return value of 1 is OK
	
public:
	NvJavaEnv();
	virtual ~NvJavaEnv();

	int LoadClass(std::string DirName, std::string ClassName);
	void UnLoadClass();
	int RunMethod(std::string Method, bool BooleanMethod = false);
	bool DoRender(NvRendererInt *Render);

};

/////////////////////////////////////////////////////////////////////////////
// NvJavaEnvList class
//
// This class manages the JAVA service list
//
class NvJavaEnvList : public NvObject
{
public:
	std::map<std::string, NvJavaEnv*> m_mList;	// list of java services
	
public:
	NvJavaEnvList();
	virtual ~NvJavaEnvList();

	// Remove all items from the list
	void RemoveAll();				
	
	// Get the NvJavaEnv corresponding to the service
	NvJavaEnv* GetJavaEnv(std::string ServiceName);
	
	// Add the NvJavaEnv corresponding to the service
	void AddJavaEnv(std::string ServiceName, NvJavaEnv* JavaEnv);
	
	// Remove the NvJavaEnv corresponding to the service
	void RemoveJavaEnv(std::string ServiceName);
};


#endif //_GLOBAL_H
