// ***************************************************************************************
// File: nidget.cpp
// Description: nidget framework
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************


#include "nvs.h"

#ifdef NV_SERVER
	#ifdef WIN32
		#ifdef _DEBUG
			#include <stdlib.h>
			#include <crtdbg.h>
			#define NVNEW new( _NORMAL_BLOCK, __FILE__, __LINE__)
		#else // _DEBUG
			#define NVNEW new
		#endif // _DEBUG
	#else	//WIN32
		#define NVNEW new
	#endif	//WIN32
#else	//NV_SERVER
	#define NVNEW new
#endif	//NV_SERVER

using namespace std;


/////////////////////////////////////////////////////////////////////////////
// NvNidgetDoc class
NvNidgetDoc::NvNidgetDoc()
{
	m_sName = "";
	m_iType = NV_NIDGET_DOC_TYPE_DYNAMIC;
	m_sLoad = "";
}

NvNidgetDoc::~NvNidgetDoc()
{
	Clear();
}

void NvNidgetDoc::Clear(bool All)
{
	if(All)
	{
		m_sName = "";
		m_sLoad = "";
		m_iType = NV_NIDGET_DOC_TYPE_DYNAMIC;
	}
}



/////////////////////////////////////////////////////////////////////////////
// NvNidget class
NvNidget::NvNidget()
{
	m_sName = "";
	m_sClass = "";
}

NvNidget::~NvNidget()
{
	Clear();
}

void NvNidget::Clear(bool All)
{
	m_vDocs.clear();
	if(All)
	{
		m_sName = "";
		m_sClass = "";
	}
}
/////////////////////////////////////////////////////////////////////////////
// NvNidgetView class
NvNidgetView::NvNidgetView()
{
	m_sName = "";
	m_iType = NV_NIDGET_VIEW_TYPE_DYNAMIC;
	m_iType2 =  NV_NIDGET_VIEW_TYPE_NORMAL;
	m_sClass = "";
	m_sParameters = "";
	m_sLabels = "";
	m_sTitle = "";
	m_sHeader = "";
	m_sFooter = "";
	m_sLanguage = "ENGLISH";
	m_sViewData = "";
	SessionIdIndex = -1;
	LanguageIndex = -1;
	m_sSectionsData = "";
}

NvNidgetView::~NvNidgetView()
{
	Clear();
}

void NvNidgetView::Clear(bool All)
{
	// remove documents
	map<string,NvNidgetDoc*>::iterator posDoc;
	for(posDoc = m_mDocs.begin(); posDoc != m_mDocs.end(); ++posDoc)
	{
		NvNidgetDoc *Doc = posDoc->second;
		if(Doc != NULL)
			delete Doc;
	}
	m_mDocs.clear();

	// remove nidgets
	map<string,NvNidget*>::iterator posNidget;
	for(posNidget = m_mNidgets.begin(); posNidget != m_mNidgets.end(); ++posNidget)
	{
		NvNidget *Nidget = posNidget->second;
		if(Nidget != NULL)
			delete Nidget;
	}
	m_mNidgets.clear();

	// Remove info and includes
	m_mInfo.clear();

	// Free section data
	m_sSectionsData = "";

	// Remove the section names
	m_vSections.clear();

	// Remove the form names
	m_mForms.clear();

	// If required, do a complete clean
	if(All)
	{
		m_sClass = "";
		m_sParameters = "";
		m_sLabels = "";
		m_sTitle = "";
		m_sHeader = "";
		m_sFooter = "";
		m_sLanguage = "ENGLISH";
		m_mParameters.clear();
		
		// Free view data
		m_sViewData = "";
		SessionIdIndex = -1;
		LanguageIndex = -1;
		m_iType = NV_NIDGET_VIEW_TYPE_DYNAMIC;
		m_iType2 =  NV_NIDGET_VIEW_TYPE_NORMAL;
	}
}



/////////////////////////////////////////////////////////////////////////////
// NvNidgetFramework class
NvNidgetFramework::NvNidgetFramework()
{
	m_sAppName = "";
	m_sFrameworkSourceDir = "";
	m_sChecksum = "";
}

NvNidgetFramework::~NvNidgetFramework()
{
	NV_LOCKOBJECT(this);
	Clear();
	NV_UNLOCKOBJECT(this);
}

void NvNidgetFramework::Clear()
{
	// remove views
	map<string,NvNidgetView*>::iterator posView;
	for(posView = m_mViews.begin(); posView != m_mViews.end(); ++posView)
	{
		NvNidgetView *View = posView->second;
		if(View != NULL)
			delete View;
	}
	m_mViews.clear();
	m_mInfo.clear();
	m_mPackages.clear();
	m_mIncludes.clear();
	m_sDebugLevel = "0";
	m_sErrorView = "";
	m_mHeaders.clear();
	m_mFooters.clear();
	m_sChecksum = "";
	m_sIcon = "";
	m_sTheme = "";
}


// Check and publish the framework if necessary
// return false in case of error and set CommandToExecute
bool NvNidgetFramework::CheckPublish(NvCommand *CommandToExecute)
{
	NV_LOCKOBJECT(this);
	GetFrameworkSourceDir();
	if(!DirExist(m_sFrameworkSourceDir.c_str()))
	{
		m_sChecksum = "";
		NV_UNLOCKOBJECT(this);
		return true;	// No publish in this case because the framework doesn't exist
	}
	
	bool bDoLoad = false;
	if(!m_mViews.size())
		bDoLoad = true;
	bool bDoPublish = false;
	string ChecksumFile = GetMain()->GetNirvaDir();
	AddDirSuffix(ChecksumFile);
	ChecksumFile += "Applications";
	AddDirSuffix(ChecksumFile);
	ChecksumFile += m_sAppName;
	AddDirSuffix(ChecksumFile);
	ChecksumFile += "Wroot";
	AddDirSuffix(ChecksumFile);
	ChecksumFile += "Framework";
	AddDirSuffix(ChecksumFile);
	ChecksumFile += "checksum.txt";
	if(!m_sChecksum.length())
	{
		// If no checksum, we verify if the checksum.txt exists on the framework dir
		if(FileExist(ChecksumFile.c_str()))
		{
			FILE *File;
			File = Nvfopen(ChecksumFile.c_str(), "r");
			if(File != NULL)
			{
				char Buffer[100];
				Buffer[0] = '\0';
				int NumRead = (int)(fread(Buffer, 1, sizeof(Buffer)-1, File));
				if(NumRead)
					Buffer[NumRead] = '\0';
				m_sChecksum = Buffer;
				fclose(File);
			}
			else
				bDoPublish = true;
		}
		else
			bDoPublish = true;
	}

	string CheckSum = "";
	if(!GetDirLastModifiedCheckSum(m_sFrameworkSourceDir, CheckSum))
	{
		if(m_sChecksum.length())
		{
			m_sChecksum = "";
			RemoveFile(ChecksumFile.c_str());
		}
		string Message = "Cannot get framework checksum";
		CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_LOAD_FRAMEWORK, Message.c_str());
		NV_UNLOCKOBJECT(this);
		return false;
	}
	
	if(m_sChecksum != CheckSum)
		bDoPublish = true;
	if(bDoPublish)
		bDoLoad = true;

	if(!bDoLoad && !bDoPublish)	// No load and no publish necessary
	{
		NV_UNLOCKOBJECT(this);
		return true;
	}

	// Do the load now

	// Clear the framework
	Clear();

	// Erase some framework wroot directories if publish required

	string WrootDir = GetMain()->GetNirvaDir();
	AddDirSuffix(WrootDir);
	WrootDir += "Applications";
	AddDirSuffix(WrootDir);
	WrootDir += m_sAppName;
	AddDirSuffix(WrootDir);
	WrootDir += "Wroot";

	string DirToRemove = WrootDir;
	AddDirSuffix(DirToRemove);
	DirToRemove += "Framework/Includes";
	if(bDoPublish)
		RemoveDir(DirToRemove.c_str());

	DirToRemove = WrootDir;
	AddDirSuffix(DirToRemove);
	DirToRemove += "Framework/Nidgets";
	if(bDoPublish)
		RemoveDir(DirToRemove.c_str());

	DirToRemove = WrootDir;
	AddDirSuffix(DirToRemove);
	DirToRemove += "Framework/Params";
	if(bDoPublish)
		RemoveDir(DirToRemove.c_str());

	DirToRemove = WrootDir;
	AddDirSuffix(DirToRemove);
	DirToRemove += "Framework/Views";
	if(bDoPublish)
		RemoveDir(DirToRemove.c_str());

	DirToRemove = WrootDir;
	AddDirSuffix(DirToRemove);
	DirToRemove += "Framework/Resources";
	if(bDoPublish)
		RemoveDir(DirToRemove.c_str());

	DirToRemove = WrootDir;
	AddDirSuffix(DirToRemove);
	DirToRemove += "Framework/Inline";
	if(bDoPublish)
		RemoveDir(DirToRemove.c_str());

	DirToRemove = WrootDir;
	AddDirSuffix(DirToRemove);
	DirToRemove += "Framework/Themes";
	if(bDoPublish)
		RemoveDir(DirToRemove.c_str());

	// First copy the fixed files
	string SourceFile = GetFrameworkSourceDir();
	AddDirSuffix(SourceFile);
	SourceFile += "System";
	AddDirSuffix(SourceFile);
	SourceFile += "jquery.js";
	if(!GetNoCaseRealFileName(SourceFile))
	{
		string Message = "file not found:";
		Message += SourceFile;
		CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_LOAD_FRAMEWORK, Message.c_str());
		Clear();
		NV_UNLOCKOBJECT(this);
		return false;
	}

	string DestinationFile = WrootDir;
	AddDirSuffix(DestinationFile);
	DestinationFile += "Framework";
	AddDirSuffix(DestinationFile);
	DestinationFile += "jquery.js";

	NvJspCleaner JspCleaner;
	if((bDoPublish) || !FileExist(DestinationFile.c_str()))
	{
		if(!CopyFileEx(SourceFile.c_str(), DestinationFile.c_str()))
		{
			string Message = "cannot copy the file:";
			Message += SourceFile;
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			Clear();
			NV_UNLOCKOBJECT(this);
			return false;
		}
	}

	SourceFile = GetFrameworkSourceDir();
	AddDirSuffix(SourceFile);
	SourceFile += "System";
	AddDirSuffix(SourceFile);
	SourceFile += "jquery.hoverIntent.minified.js";
	if(!GetNoCaseRealFileName(SourceFile))
	{
		string Message = "file not found:";
		Message += SourceFile;
		CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_LOAD_FRAMEWORK, Message.c_str());
		Clear();
		NV_UNLOCKOBJECT(this);
		return false;
	}

	DestinationFile = WrootDir;
	AddDirSuffix(DestinationFile);
	DestinationFile += "Framework";
	AddDirSuffix(DestinationFile);
	DestinationFile += "jquery.hoverIntent.minified.js";


	if((bDoPublish) || !FileExist(DestinationFile.c_str()))
	{
//		if(!CopyFileEx(SourceFile.c_str(), DestinationFile.c_str()))
		if(!JspCleaner.DoClean(SourceFile.c_str(), DestinationFile.c_str()))
		{
			string Message = "cannot copy the file:";
			Message += SourceFile;
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			Clear();
			NV_UNLOCKOBJECT(this);
			return false;
		}
	}


	SourceFile = GetFrameworkSourceDir();
	AddDirSuffix(SourceFile);
	SourceFile += "System";
	AddDirSuffix(SourceFile);
	SourceFile += "nvfmw.js";
	if(!GetNoCaseRealFileName(SourceFile))
	{
		string Message = "file not found:";
		Message += SourceFile;
		CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_LOAD_FRAMEWORK, Message.c_str());
		Clear();
		NV_UNLOCKOBJECT(this);
		return false;
	}

	DestinationFile = WrootDir;
	AddDirSuffix(DestinationFile);
	DestinationFile += "Framework";
	AddDirSuffix(DestinationFile);
	DestinationFile += "nvfmw.js";


	if((bDoPublish) || !FileExist(DestinationFile.c_str()))
	{
//		if(!CopyFileEx(SourceFile.c_str(), DestinationFile.c_str()))
		if(!JspCleaner.DoClean(SourceFile.c_str(), DestinationFile.c_str()))
		{
			string Message = "cannot copy the file:";
			Message += SourceFile;
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			Clear();
			NV_UNLOCKOBJECT(this);
			return false;
		}
	}

	SourceFile = GetFrameworkSourceDir();
	AddDirSuffix(SourceFile);
	SourceFile += "System";
	AddDirSuffix(SourceFile);
	SourceFile += "nvajax.js";
	if(!GetNoCaseRealFileName(SourceFile))
	{
		string Message = "file not found:";
		Message += SourceFile;
		CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_LOAD_FRAMEWORK, Message.c_str());
		Clear();
		NV_UNLOCKOBJECT(this);
		return false;
	}

	DestinationFile = WrootDir;
	AddDirSuffix(DestinationFile);
	DestinationFile += "Framework";
	AddDirSuffix(DestinationFile);
	DestinationFile += "nvajax.js";

	if((bDoPublish) || !FileExist(DestinationFile.c_str()))
	{
//		if(!CopyFileEx(SourceFile.c_str(), DestinationFile.c_str()))
		if(!JspCleaner.DoClean(SourceFile.c_str(), DestinationFile.c_str()))
		{
			string Message = "cannot copy the file:";
			Message += SourceFile;
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			Clear();
			NV_UNLOCKOBJECT(this);
			return false;
		}
	}


	// Load the framework config file
	string XsltFile = GetMain()->GetNirvaDir();
	XsltFile += "Files/System/Framework/get_framework.xsl";
	if(!GetNoCaseRealFileName(XsltFile))
	{
		CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_LOAD_FRAMEWORK, "get_framework.xsl not found");
		Clear();
		NV_UNLOCKOBJECT(this);
		return false;
	}

	NvContainer Container;
	string FrameworkSource = GetFrameworkSourceDir();
	AddDirSuffix(FrameworkSource);
	FrameworkSource += "config.xml";

	if(!GetNoCaseRealFileName(FrameworkSource))
	{
		string Message = "file not found:";
		Message += FrameworkSource;
		CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_LOAD_FRAMEWORK, Message.c_str());
		Clear();
		NV_UNLOCKOBJECT(this);
		return false;
	}

	NvObjBinary Binary;
	if(!Binary.Load(FrameworkSource.c_str()))
	{
		string Message = "cannot load file:";
		Message += FrameworkSource;
		CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_LOAD_FRAMEWORK, Message.c_str());
		Clear();
		NV_UNLOCKOBJECT(this);
		return false;
	}

	char *Buffer = Binary.GetBuffer();
	unsigned long BufferSize = Binary.m_isize;
	Binary.m_cdata = NULL;
	Binary.m_isize = 0;

	// Parse the config
	NvXml XmlParser;
	if(!XmlParser.XsltTransform(&Buffer, &BufferSize, XsltFile.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_XSL_PROCESSING_ERROR);
		if(Buffer != NULL)
			free(Buffer);
		Clear();
		NV_UNLOCKOBJECT(this);
		return false;
	}

	// Get input
	string NameSpace;
	int XmlCoding = XML_MODEL_SIMPLE;
	if(!XmlParser.SetInputData(&Container, Buffer, BufferSize, CommandToExecute, NameSpace, XmlCoding))
	{
		string Message = "cannot get framework input data from file:";
		Message += FrameworkSource;
		CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_LOAD_FRAMEWORK, Message.c_str());
		if(Buffer != NULL)
			free(Buffer);
		Clear();
		NV_UNLOCKOBJECT(this);
		return false;
	}
	
	if(Buffer != NULL)
		free(Buffer);

	NvObjIndStringList* MyIndStringList = (NvObjIndStringList *)(Container.GetObject("info"));
	if(MyIndStringList != NULL)
	{
		if(!GetMain()->b_Unicode)
			MyIndStringList->Encode(false, true);
		map<string,string>::iterator pos;
		for(pos = MyIndStringList->m_mList.begin(); pos != MyIndStringList->m_mList.end(); ++pos)
			m_mInfo[pos->first] = pos->second;
	}
	
	NvObjString *MyString = (NvObjString *)(Container.GetObject("debug"));
	if(MyString != NULL)
	{
		m_sDebugLevel = MyString->GetValue();
		if(!m_sDebugLevel.length())
			m_sDebugLevel = "0";
	}
	
	MyString = (NvObjString *)(Container.GetObject("error"));
	if(MyString != NULL)
	{
		m_sErrorView = MyString->GetValue();
		MakeLower(m_sErrorView);
	}
	
	MyString = (NvObjString *)(Container.GetObject("icon"));
	if(MyString != NULL)
		m_sIcon = MyString->GetValue();
	
	MyString = (NvObjString *)(Container.GetObject("theme"));
	if(MyString != NULL)
		m_sTheme = MyString->GetValue();

	NvObjStringList *MyStringList = (NvObjStringList *)(Container.GetObject("includes"));
	if(MyStringList != NULL)
	{
		if(!GetMain()->b_Unicode)
			MyStringList->Encode(false, true);
		int i;
		for(i=0;i<MyStringList->GetSize();i++)
			m_mIncludes[MyStringList->GetValue(i)] = "";
	}

	MyStringList = (NvObjStringList *)(Container.GetObject("headers"));
	if(MyStringList != NULL)
	{
		if(!GetMain()->b_Unicode)
			MyStringList->Encode(false, true);
		int i;
		for(i=0;i<MyStringList->GetSize();i++)
			m_mHeaders[MyStringList->GetValue(i)] = "";
	}

	MyStringList = (NvObjStringList *)(Container.GetObject("footers"));
	if(MyStringList != NULL)
	{
		if(!GetMain()->b_Unicode)
			MyStringList->Encode(false, true);
		int i;
		for(i=0;i<MyStringList->GetSize();i++)
			m_mFooters[MyStringList->GetValue(i)] = "";
	}

	MyStringList = (NvObjStringList *)(Container.GetObject("nidget_packages"));
	if(MyStringList != NULL)
	{
		if(!GetMain()->b_Unicode)
			MyStringList->Encode(false, true);
		int i;
		for(i=0;i<MyStringList->GetSize();i++)
			m_mPackages[MyStringList->GetValue(i)] = "";
	}
	
	MyStringList = (NvObjStringList *)(Container.GetObject("views"));
	if(MyStringList != NULL)
	{
		// The views must be loaded in the following order: headers, footers and then other views
		if(!GetMain()->b_Unicode)
			MyStringList->Encode(false, true);
		vector<string> ViewsToLoad;
		int i;
		// Headers first
		for(i=0;i<MyStringList->GetSize();i++)
		{
			string ViewName = MyStringList->GetValue(i);
			MakeLower(ViewName);
			if(m_mHeaders.find(ViewName) != m_mHeaders.end())
				ViewsToLoad.push_back(ViewName);
		}
		// Then footers
		for(i=0;i<MyStringList->GetSize();i++)
		{
			string ViewName = MyStringList->GetValue(i);
			MakeLower(ViewName);
			if(m_mFooters.find(ViewName) != m_mFooters.end())
				ViewsToLoad.push_back(ViewName);
		}
		// Then others
		for(i=0;i<MyStringList->GetSize();i++)
		{
			string ViewName = MyStringList->GetValue(i);
			MakeLower(ViewName);
			if((m_mHeaders.find(ViewName) == m_mHeaders.end()) && (m_mFooters.find(ViewName) == m_mFooters.end()))
				ViewsToLoad.push_back(ViewName);
		}
		for(i=0;i<(int)(ViewsToLoad.size());i++)
		{
			string ViewName = ViewsToLoad[i];
			if(m_mViews.find(ViewName) != m_mViews.end())
				continue;	// The view is already loaded

			NvNidgetView* View = new NvNidgetView();
			View->m_sName = ViewName;
			if(!LoadView(View, CommandToExecute, bDoPublish))
			{
				delete View;
				Clear();
				NV_UNLOCKOBJECT(this);
				return false;
			}
			
			// Build at least the english parameters
			string LanguageHere = View->m_sLanguage;
			if((View->m_iType == NV_NIDGET_VIEW_TYPE_DYNAMIC) && (CommandToExecute != NULL))
				LanguageHere = CommandToExecute->m_sLanguage;
			CleanString(LanguageHere);
			MakeUpper(LanguageHere);
			if(!LanguageHere.length())
				LanguageHere = "ENGLISH";
			if(!BuildViewParameters(View, CommandToExecute, LanguageHere))
			{
				delete View;
				Clear();
				NV_UNLOCKOBJECT(this);
				return false;
			}
			m_mViews[ViewName] = View;
		}
	}

	// Verify that the error view exists and is a static view
	if(m_sErrorView.length())
	{
		if(m_mViews.find(m_sErrorView) == m_mViews.end())
		{
			string Message = "The error view ";
			Message += m_sErrorView;
			Message += " is not declared in file ";
			Message += FrameworkSource;
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_LOAD_FRAMEWORK, Message.c_str());
			Clear();
			NV_UNLOCKOBJECT(this);
			return false;
		}
		
		NvNidgetView* View = m_mViews[m_sErrorView];
		if(View->m_iType != NV_NIDGET_VIEW_TYPE_STATIC)
		{
			string Message = "The error view ";
			Message += m_sErrorView;
			Message += " is not a static view";
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_LOAD_FRAMEWORK, Message.c_str());
			Clear();
			NV_UNLOCKOBJECT(this);
			return false;
		}
	}

	// Publish the Nidget packages
	map<string,string>::iterator pos;
	for(pos = m_mPackages.begin(); pos != m_mPackages.end(); ++pos)
	{
		SourceFile = GetFrameworkSourceDir();
		AddDirSuffix(SourceFile);
		SourceFile += "Nidgets";
		AddDirSuffix(SourceFile);
		SourceFile += pos->first;
		if(!GetNoCaseRealFileName(SourceFile))
		{
			string Message = "cannot get nidget source file:";
			Message += SourceFile;
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_LOAD_FRAMEWORK, Message.c_str());
			Clear();
			NV_UNLOCKOBJECT(this);
			return false;
		}
		string PublishDirName = WrootDir;
		AddDirSuffix(PublishDirName);
		PublishDirName += "Framework";
		AddDirSuffix(PublishDirName);
		PublishDirName += "Nidgets";
		AddDirSuffix(PublishDirName);
		string::size_type index = SourceFile.find_last_of("/\\");
		string OnlyFileName = SourceFile.substr(index+1);
		string OnlyDir = SourceFile.substr(GetFrameworkSourceDir().length()+9);
		index = OnlyDir.find_last_of("/\\");
		OnlyDir = OnlyDir.substr(0, index);
		PublishDirName += OnlyDir;
		if(!CreateDir(PublishDirName.c_str(), false))
		{
			string Message = "cannot create directory:";
			Message += PublishDirName;
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_LOAD_FRAMEWORK, Message.c_str());
			Clear();
			NV_UNLOCKOBJECT(this);
			return false;
		}
		string PublishFileName = PublishDirName;
		AddDirSuffix(PublishFileName);
		PublishFileName += OnlyFileName;
		if((bDoPublish) || !FileExist(PublishFileName.c_str()))
		{
//			if(!CopyFileEx(SourceFile.c_str(), PublishFileName.c_str()))
			if(!JspCleaner.DoClean(SourceFile.c_str(), PublishFileName.c_str()))
			{
				string Message = "cannot create the file:";
				Message += PublishFileName;
				CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_LOAD_FRAMEWORK, Message.c_str());
				Clear();
				NV_UNLOCKOBJECT(this);
				return false;
			}
		}
		// Check for Nidget static resources (in subdir nidgetsourcefile_res)
		string SrcResDir = SourceFile;
		index = SrcResDir.rfind('.');
		if(index != string::npos)
			SrcResDir = SrcResDir.substr(0, index);
		SrcResDir += "_res";
		if(DirExist(SrcResDir.c_str()))
		{
			// There is a resource directory
			string DstResDir = PublishDirName;
			AddDirSuffix(DstResDir);
			index = OnlyFileName.rfind('.');
			if(index == string::npos)
				DstResDir += OnlyFileName;
			else
				DstResDir += OnlyFileName.substr(0, index);
			DstResDir += "_res";
			if((bDoPublish) || !DirExist(DstResDir.c_str()))
			{
				RemoveDir(DstResDir.c_str());
				if(!CopyDir(SrcResDir.c_str(), DstResDir.c_str()))
				{
					string Message = "cannot copy nidget resource directory:";
					Message += pos->first;
					CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_LOAD_FRAMEWORK, Message.c_str());
					Clear();
					NV_UNLOCKOBJECT(this);
					return false;
				}
			}
		}
	}

	// Publish the includes
	for(pos = m_mIncludes.begin(); pos != m_mIncludes.end(); ++pos)
	{
		string SourceFile = GetFrameworkSourceDir();
		AddDirSuffix(SourceFile);
		SourceFile += "Includes";
		AddDirSuffix(SourceFile);
		SourceFile += pos->first;
		if(!GetNoCaseRealFileName(SourceFile))
		{
			string Message = "cannot get include file:";
			Message += SourceFile;
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_LOAD_FRAMEWORK, Message.c_str());
			Clear();
			NV_UNLOCKOBJECT(this);
			return false;
		}

		string PublishFileName = WrootDir;
		AddDirSuffix(PublishFileName);
		PublishFileName += "Framework";
		AddDirSuffix(PublishFileName);
		PublishFileName += "Includes";
		if(!CreateDir(PublishFileName.c_str(), false))	// Create the dir if it doesn't exist
		{
			string Message = "cannot create directory:";
			Message += PublishFileName;
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_LOAD_FRAMEWORK, Message.c_str());
			Clear();
			NV_UNLOCKOBJECT(this);
			return false;
		}
		AddDirSuffix(PublishFileName);
		PublishFileName += pos->first;
		if((bDoPublish) || !FileExist(PublishFileName.c_str()))
		{
			if(!CopyFileEx(SourceFile.c_str(), PublishFileName.c_str()))
			{
				string Message = "cannot create the file:";
				Message += PublishFileName;
				CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_LOAD_FRAMEWORK, Message.c_str());
				Clear();
				NV_UNLOCKOBJECT(this);
				return false;
			}
		}
	}

	// Publish the resources
	string ResourceSrcDir = GetFrameworkSourceDir();
	AddDirSuffix(ResourceSrcDir);
	ResourceSrcDir += "Resources";
	if(DirExist(ResourceSrcDir.c_str()))
	{
		string ResourceDstDir = WrootDir;
		AddDirSuffix(ResourceDstDir);
		ResourceDstDir += "Framework";
		AddDirSuffix(ResourceDstDir);
		ResourceDstDir += "Resources";
		if((bDoPublish) || !DirExist(ResourceDstDir.c_str()))
		{
			RemoveDir(ResourceDstDir.c_str(), false);
			CreateDir(ResourceDstDir.c_str());
			if(!CopyDir(ResourceSrcDir.c_str(), ResourceDstDir.c_str()))
			{
				string Message = "Cannot copy resource directory";
				Message += ResourceSrcDir;
				CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_LOAD_FRAMEWORK, Message.c_str());
				Clear();
				NV_UNLOCKOBJECT(this);
				return false;
			}
		}
	}


	// Publish the theme
	if(m_sTheme.length())
	{
		SourceFile = GetFrameworkSourceDir();
		AddDirSuffix(SourceFile);
		SourceFile += "Themes";
		AddDirSuffix(SourceFile);
		SourceFile += m_sTheme;
		if(m_sTheme.rfind(".css") == string::npos)
			SourceFile += ".css";

		DestinationFile = WrootDir;
		AddDirSuffix(DestinationFile);
		DestinationFile += "Framework";
		AddDirSuffix(DestinationFile);
		DestinationFile += "Themes";
		if(!CreateDir(DestinationFile.c_str(), false))
		{
			string Message = "cannot create directory:";
			Message += DestinationFile;
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_LOAD_FRAMEWORK, Message.c_str());
			Clear();
			NV_UNLOCKOBJECT(this);
			return false;
		}
		AddDirSuffix(DestinationFile);
		DestinationFile += m_sTheme;
		if(m_sTheme.rfind(".css") == string::npos)
			DestinationFile += ".css";
		if((bDoPublish) || !FileExist(DestinationFile.c_str()))
		{
			if(!CopyFileEx(SourceFile.c_str(), DestinationFile.c_str()))
			{
				string Message = "cannot create the file:";
				Message += DestinationFile;
				CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_LOAD_FRAMEWORK, Message.c_str());
				Clear();
				NV_UNLOCKOBJECT(this);
				return false;
			}
		}
	}

	// Partially clears the view (in order to save memory)
	map<string,NvNidgetView*>::iterator posView;
	for(posView = m_mViews.begin(); posView != m_mViews.end(); ++posView)
	{
		NvNidgetView *View = posView->second;
		if(View != NULL)
			View->Clear(false);
	}

	m_sChecksum = CheckSum;
	FILE *File;
	File = Nvfopen(ChecksumFile.c_str(), "wb");
	if(File != NULL)
	{
		fwrite(m_sChecksum.c_str(), 1, m_sChecksum.length(), File);
		fclose(File);
	}

	NV_UNLOCKOBJECT(this);
	return true;
}



// Main framework entry point
// SessionId is the current session ID
// CommandToExecute is the current command. It's used here only for error reporting
// Result is an array containing the resulting view data with the SessionId, this is directly sent back to browser
// ResultSize is the size of the allocated result buffer
// ForceCheck is used in debug mode and allows to force the reload of the view if necessary
// Error view is set when calling a view for displaying error
bool NvNidgetFramework::GetView(const char* Name, const char* SessionId, NvCommand *CommandToExecute, char** Result, unsigned long* ResultSize, bool ForceCheck, bool ErrorView)
{
	string SessionIdHere = SessionId;
	string NameHere = Name;
	CleanString(NameHere);
	MakeLower(NameHere);
	if(!NameHere.length() && ErrorView)
	{
		NameHere = m_sErrorView;
		if(!NameHere.length())
			return false;
	}
	string::size_type index = NameHere.rfind('/');
	if(index != string::npos)
		NameHere = NameHere.substr(index+1);
	index = NameHere.rfind('.');
	if(index != string::npos)
		NameHere = NameHere.substr(0, index);


	bool bDoCheck = ForceCheck;
	NvNidgetView *View = NULL;
	NV_LOCKOBJECT(this);
	if(m_mViews.find(NameHere) == m_mViews.end())
		bDoCheck = true;
	else
	{
		View = m_mViews[NameHere];
		if(!View->m_sViewData.length())
			bDoCheck = true;
	}
	if(bDoCheck)
	{
		NV_UNLOCKOBJECT(this);
		if(!CheckPublish(CommandToExecute))
			return false;
		NV_LOCKOBJECT(this);
		if(m_mViews.find(NameHere) == m_mViews.end())
		{
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_VIEW_NOT_FOUND, NameHere);
			NV_UNLOCKOBJECT(this);
			return false;
		}
		View = m_mViews[NameHere];
	}

	// The view data is now in memory
	// We work on a copy in order to save time
	string OutputData = View->m_sViewData;
	int SessionIdIndex = View->SessionIdIndex;
	int LanguageIndex = View->LanguageIndex;
	string LanguageHere = View->m_sLanguage;
	if((CommandToExecute != NULL) && CommandToExecute->m_sLanguage.length())
		LanguageHere = CommandToExecute->m_sLanguage;
	CleanString(LanguageHere);
	MakeUpper(LanguageHere);
	if(!LanguageHere.length())
		LanguageHere = "ENGLISH";

	// Now Check if the parameter file has to be rebuild
	string ViewParamFile = View->m_sParameters;
	if(ViewParamFile.length())
	{
		// The View->->m_mParameters member contains the list of parameters files already built
		// So we don't do the check if the file has been built and if we are not forced to do the check
		// This feature is to save time
		if(ForceCheck || (View->m_mParameters.find(LanguageHere) == View->m_mParameters.end()))
		{
			View->m_mParameters.erase(LanguageHere);
			// Now we do the check
			bool bBuildParam = CheckViewParameters(NameHere, LanguageHere);

			if(bBuildParam)
			{
				if(!BuildViewParameters(View, CommandToExecute, LanguageHere))
				{
					NV_UNLOCKOBJECT(this);
					return false;

				}
			}
			View->m_mParameters[LanguageHere] = "";
		}
	}

	// Check if header parameters have to be rebuilt
	NvNidgetView* Header = NULL;
	if(View->m_sHeader.length() && (m_mViews.find(View->m_sHeader) != m_mViews.end()))
		Header = m_mViews[View->m_sHeader];
	if(Header != NULL)
	{
		ViewParamFile = Header->m_sParameters;
		if(ViewParamFile.length())
		{
			// The Header->m_mParameters member contains the list of parameters files already built
			// So we don't do the check if the file has been built and if we are not forced to do the check
			// This feature is to save time
			if(ForceCheck || (Header->m_mParameters.find(LanguageHere) == Header->m_mParameters.end()))
			{
				Header->m_mParameters.erase(LanguageHere);
				// Now we do the check
				bool bBuildParam = CheckViewParameters(Header->m_sName, LanguageHere);

				if(bBuildParam)
				{
					if(!BuildViewParameters(Header, CommandToExecute, LanguageHere))
					{
						NV_UNLOCKOBJECT(this);
						return false;

					}
				}
				Header->m_mParameters[LanguageHere] = "";
			}
		}
	}

	// Check if footer parameters have to be rebuilt
	NvNidgetView* Footer = NULL;
	if(View->m_sFooter.length() && (m_mViews.find(View->m_sFooter) != m_mViews.end()))
		Footer = m_mViews[View->m_sFooter];
	if(Footer != NULL)
	{
		ViewParamFile = Footer->m_sParameters;
		if(ViewParamFile.length())
		{
			// The Footer->m_mParameters member contains the list of parameters files already built
			// So we don't do the check if the file has been built and if we are not forced to do the check
			// This feature is to save time
			if(ForceCheck || (Footer->m_mParameters.find(LanguageHere) == Footer->m_mParameters.end()))
			{
				Footer->m_mParameters.erase(LanguageHere);
				// Now we do the check
				bool bBuildParam = CheckViewParameters(Footer->m_sName, LanguageHere);

				if(bBuildParam)
				{
					if(!BuildViewParameters(Footer, CommandToExecute, LanguageHere))
					{
						NV_UNLOCKOBJECT(this);
						return false;

					}
				}
				Footer->m_mParameters[LanguageHere] = "";
			}
		}
	}


	NV_UNLOCKOBJECT(this);

	// If this is an error view, we must resolve error variables
	if(ErrorView)
	{
		string ErrorService = CommandToExecute->Error.m_sErrorService;
		if(!GetMain()->b_Unicode)
			EncodeString(ErrorService, false, true);
		string ErrorClass = CommandToExecute->Error.m_sErrorClass;
		if(!GetMain()->b_Unicode)
			EncodeString(ErrorClass, false, true);
		char Buff[20];
		sprintf(Buff, "%d", CommandToExecute->Error.m_iErrorCode);
		string ErrorCode = Buff;
		string ErrorDesc = CommandToExecute->Error.GetErrorDesc(LanguageHere.c_str());
		if(!GetMain()->b_Unicode)
			EncodeString(ErrorDesc, false, true);
		string ErrorInfo = CommandToExecute->Error.m_sErrorInfo;
		if(!GetMain()->b_Unicode)
			EncodeString(ErrorInfo, false, true);

		string::size_type index = OutputData.find("$$$NV$$$ERRORVIEW$$$");
		if(index != string::npos)
			OutputData.replace(index, 20, "YES");
		index = OutputData.find("$$$NV$$$ERRORSERVICE$$$");
		if(index != string::npos)
			OutputData.replace(index, 23, ErrorService);
		index = OutputData.find("$$$NV$$$ERRORCLASS$$$");
		if(index != string::npos)
			OutputData.replace(index, 21, ErrorClass);
		index = OutputData.find("$$$NV$$$ERRORCODE$$$");
		if(index != string::npos)
			OutputData.replace(index, 20, ErrorCode);
		index = OutputData.find("$$$NV$$$ERRORDESCRIPTION$$$");
		if(index != string::npos)
			OutputData.replace(index, 27, ErrorDesc);
		index = OutputData.find("$$$NV$$$ERRORINFO$$$");
		if(index != string::npos)
			OutputData.replace(index, 20, ErrorInfo);
	}

	// We must set the output and resolve the session ID and language
	unsigned long OutputSize = (unsigned long)(OutputData.length());
	bool bInsertSessionId = false;
	if(SessionIdIndex > 0)
	{
		bInsertSessionId = true;
		OutputSize += (unsigned long)(SessionIdHere.length());
	}


	OutputSize += (unsigned long)(LanguageHere.length());

	*Result = (char *)malloc(OutputSize);
	*ResultSize = OutputSize;
	if(*Result == NULL)
	{
		CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_NOT_ENOUGH_MEMORY, NameHere);
		return false;
	}

	// Insert Language
	// Language index must be before sessionId index
	char *ptr = *Result;
	memcpy(ptr, OutputData.c_str(), LanguageIndex);
	ptr += LanguageIndex;
	memcpy(ptr, LanguageHere.c_str(), LanguageHere.length());
	ptr += LanguageHere.length();
	
	if(bInsertSessionId)
	{
		memcpy(ptr, OutputData.c_str() + LanguageIndex, SessionIdIndex - LanguageIndex);
		ptr += SessionIdIndex - LanguageIndex;
		memcpy(ptr, SessionIdHere.c_str(), SessionIdHere.length());
		ptr += SessionIdHere.length();
		memcpy(ptr, OutputData.c_str() + SessionIdIndex, OutputData.length() - SessionIdIndex);
	}
	else
	{
		// In this case, we just copy the content of the view to the ouput
		memcpy(ptr, OutputData.c_str() + LanguageIndex, OutputData.length() - LanguageIndex);
	}

	return true;
}

// Check if the view is static or not
bool NvNidgetFramework::IsStaticView(const char* Name, bool ForceCheck)
{
	string NameHere = Name;
	CleanString(NameHere);
	MakeLower(NameHere);
	string::size_type index = NameHere.rfind('/');
	if(index != string::npos)
		NameHere = NameHere.substr(index+1);
	index = NameHere.rfind('.');
	if(index != string::npos)
		NameHere = NameHere.substr(0, index);


	bool bDoCheck = ForceCheck;
	NvNidgetView *View = NULL;
	NV_LOCKOBJECT(this);
	if(m_mViews.find(NameHere) == m_mViews.end())
		bDoCheck = true;
	else
		View = m_mViews[NameHere];
	if(bDoCheck)
	{
		NV_UNLOCKOBJECT(this);
		NvCommand CommandToExecute;
		if(!CheckPublish(&CommandToExecute))
			return false;
		NV_LOCKOBJECT(this);
		if(m_mViews.find(NameHere) == m_mViews.end())
		{
			NV_UNLOCKOBJECT(this);
			return true;	// An unexisting view is considered to be static (in order to avoid Nirva issuing a session open)
		}
		View = m_mViews[NameHere];
	}
	
	bool RetVal = false;
	if(View->m_iType == NV_NIDGET_VIEW_TYPE_STATIC)
		RetVal = true;
	if(View->m_iType == NV_NIDGET_VIEW_TYPE_HTML)
		RetVal = true;
	
	NV_UNLOCKOBJECT(this);

	return RetVal;
}



bool NvNidgetFramework::LoadView(NvNidgetView* View, NvCommand *CommandToExecute, bool bDoPublish)
{
	// First Clear completly the view (except view name)
	View->Clear();

	// remove all the parameter files (one file per language)
	string WrootDir = GetMain()->GetNirvaDir();
	AddDirSuffix(WrootDir);
	WrootDir += "Applications";
	AddDirSuffix(WrootDir);
	WrootDir += m_sAppName;
	AddDirSuffix(WrootDir);
	WrootDir += "Wroot";

	string ParamDir = WrootDir;
	AddDirSuffix(ParamDir);
	ParamDir += "Framework";
	AddDirSuffix(ParamDir);
	ParamDir += "Params";

	string ParamFiles = View->m_sName;
	ParamFiles += "_params_";

	if(bDoPublish)
		RemoveDir(ParamDir.c_str(), false, ParamFiles.c_str(), NULL);

	bool isHeader = false;
	if(m_mHeaders.find(View->m_sName) != m_mHeaders.end())
		isHeader = true;

	bool isFooter = false;
	if(m_mFooters.find(View->m_sName) != m_mFooters.end())
		isFooter = true;

	// Then load the view
	NvXml XmlParser;
	string XsltFile = GetMain()->GetNirvaDir();
	XsltFile += "Files/System/Framework/get_view.xsl";
	if(!GetNoCaseRealFileName(XsltFile))
	{
		CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, "get_view.xsl not found");
		View->Clear();
		return false;
	}

	NvContainer Container;
	string NameSpace;
	int XmlCoding = XML_MODEL_SIMPLE;
	string ViewSource = GetFrameworkSourceDir();
	AddDirSuffix(ViewSource);
	ViewSource += "Views";
	AddDirSuffix(ViewSource);
	ViewSource += View->m_sName;
	ViewSource += ".xml";

	if(!GetNoCaseRealFileName(ViewSource))
	{
		string Message = "file not found:";
		Message += ViewSource;
		CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
		View->Clear();
		return false;
	}

	NvObjBinary Binary;
	if(!Binary.Load(ViewSource.c_str()))
	{
		string Message = "cannot load file:";
		Message += ViewSource;
		CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
		View->Clear();
		return false;
	}

	char *Buffer = Binary.GetBuffer();
	unsigned long BufferSize = Binary.m_isize;
	Binary.m_cdata = NULL;
	Binary.m_isize = 0;

	// Parse the view
	if(!XmlParser.XsltTransform(&Buffer, &BufferSize, XsltFile.c_str()))
	{
		CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_XSL_PROCESSING_ERROR);
		if(Buffer != NULL)
			free(Buffer);
		View->Clear();
		return false;
	}

	// Get input
	if(!XmlParser.SetInputData(&Container, Buffer, BufferSize, CommandToExecute, NameSpace, XmlCoding))
	{
		string Message = "cannot get view input data from file:";
		Message += ViewSource;
		CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
		if(Buffer != NULL)
			free(Buffer);
		View->Clear();
		return false;
	}
	
	if(Buffer != NULL)
		free(Buffer);

	NvObjIndStringList *MyIndStringList = (NvObjIndStringList *)(Container.GetObject("info"));
	if(MyIndStringList != NULL)
	{
		if(!GetMain()->b_Unicode)
			MyIndStringList->Encode(false, true);
		map<string,string>::iterator pos;
		for(pos = MyIndStringList->m_mList.begin(); pos != MyIndStringList->m_mList.end(); ++pos)
			View->m_mInfo[pos->first] = pos->second;
	}


	NvObjString *MyString = (NvObjString *)(Container.GetObject("type"));
	if(MyString != NULL)
	{
		if(!GetMain()->b_Unicode)
			MyString->Encode(false, true);
		string ViewType = MyString->GetValue();
		MakeLower(ViewType);
		CleanString(ViewType);
		if(ViewType == "static")
			View->m_iType = NV_NIDGET_VIEW_TYPE_STATIC;
		else if(ViewType == "html")
			View->m_iType = NV_NIDGET_VIEW_TYPE_HTML;
		else
			View->m_iType = NV_NIDGET_VIEW_TYPE_DYNAMIC;
	}

	if(View->m_iType != NV_NIDGET_VIEW_TYPE_HTML)
	{
		MyString = (NvObjString *)(Container.GetObject("class"));
		if(MyString != NULL)
		{
			if(!GetMain()->b_Unicode)
				MyString->Encode(false, true);
			View->m_sClass = MyString->GetValue();
		}
		
		MyString = (NvObjString *)(Container.GetObject("parameters"));
		if(MyString != NULL)
		{
			if(!GetMain()->b_Unicode)
				MyString->Encode(false, true);
			View->m_sParameters = MyString->GetValue();
		}

		MyString = (NvObjString *)(Container.GetObject("labels"));
		if(MyString != NULL)
		{
			if(!GetMain()->b_Unicode)
				MyString->Encode(false, true);
			View->m_sLabels = MyString->GetValue();
		}
	}

	MyString = (NvObjString *)(Container.GetObject("title"));
	if(MyString != NULL)
	{
		if(!GetMain()->b_Unicode)
			MyString->Encode(false, true);
		View->m_sTitle = MyString->GetValue();
	}


	NvNidgetView* Header = NULL;
	NvNidgetView* Footer = NULL;

	if(View->m_iType == NV_NIDGET_VIEW_TYPE_HTML)
	{
		if(isHeader || isFooter)
		{
			string Message = "The view ";
			Message += View->m_sName;
			Message += " is an html view and cannot be a header or footer";
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			View->Clear();
			return false;
		}
	}
	
	if(View->m_iType != NV_NIDGET_VIEW_TYPE_HTML)
	{
		if(!isHeader && !isFooter)
		{
			// A header or footer cannot have itself a header or a footer
			// So we get the header and footer name only for normal views
			MyString = (NvObjString *)(Container.GetObject("header"));
			if(MyString != NULL)
			{
				if(!GetMain()->b_Unicode)
					MyString->Encode(false, true);
				View->m_sHeader = MyString->GetValue();
			}

			MyString = (NvObjString *)(Container.GetObject("footer"));
			if(MyString != NULL)
			{
				if(!GetMain()->b_Unicode)
					MyString->Encode(false, true);
				View->m_sFooter = MyString->GetValue();
			}
		}
	}

	if(View->m_sHeader.length())
	{
		if(m_mHeaders.find(View->m_sHeader) == m_mHeaders.end())
		{
			string Message = "The header ";
			Message += View->m_sHeader;
			Message += " used in view ";
			Message += View->m_sName;
			Message += " is not defined in the header list";
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			View->Clear();
			return false;
		}
		if(m_mViews.find(View->m_sHeader) == m_mViews.end())
		{
			string Message = "The header view ";
			Message += View->m_sHeader;
			Message += " used in view ";
			Message += View->m_sName;
			Message += " is not defined in the view list";
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			View->Clear();
			return false;
		}
		Header = m_mViews[View->m_sHeader];
	}

	if((Header != NULL) && (View->m_iType == NV_NIDGET_VIEW_TYPE_STATIC) && (Header->m_iType == NV_NIDGET_VIEW_TYPE_DYNAMIC))
	{
		string Message = "The static view ";
		Message += View->m_sName;
		Message += " is using a dynamic header(";
		Message += View->m_sHeader;
		Message += ")";
		CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
		View->Clear();
		return false;
	}

	if(View->m_sFooter.length())
	{
		if(m_mFooters.find(View->m_sFooter) == m_mFooters.end())
		{
			string Message = "The footer ";
			Message += View->m_sFooter;
			Message += " used in view ";
			Message += View->m_sName;
			Message += " is not defined in the footer list";
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			View->Clear();
			return false;
		}
		if(m_mViews.find(View->m_sFooter) == m_mViews.end())
		{
			string Message = "The footer view ";
			Message += View->m_sFooter;
			Message += " used in view ";
			Message += View->m_sName;
			Message += " is not defined in the view list";
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			View->Clear();
			return false;
		}
		Footer = m_mViews[View->m_sFooter];
	}

	if((Footer != NULL) && (View->m_iType == NV_NIDGET_VIEW_TYPE_STATIC) && (Footer->m_iType == NV_NIDGET_VIEW_TYPE_DYNAMIC))
	{
		string Message = "The static view ";
		Message += View->m_sName;
		Message += " is using a dynamic footer(";
		Message += View->m_sFooter;
		Message += ")";
		CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
		View->Clear();
		return false;
	}

	MyString = (NvObjString *)(Container.GetObject("language"));
	if(MyString != NULL)
	{
		if(!GetMain()->b_Unicode)
			MyString->Encode(false, true);
		View->m_sLanguage = MyString->GetValue();
	}

	NvObjTable *MyTable = (NvObjTable *)(Container.GetObject("docs"));
	if(View->m_iType != NV_NIDGET_VIEW_TYPE_HTML)
	{
		if(MyTable != NULL)
		{
			if(!GetMain()->b_Unicode)
				MyTable->Encode(false, true);
			int NumRows = MyTable->GetNumRow();
			int i;
			for(i=0;i<NumRows;i++)
			{
				string DocName = MyTable->GetCellLine(i, 0);
				CleanString(DocName, true);
				MakeLower(DocName);
				string DocType = MyTable->GetCellLine(i, 1);
				CleanString(DocType, true);
				string DocLoad = MyTable->GetCellLine(i, 2);
				// Check no name
				if(!DocName.length())
				{
					string Message = "The view ";
					Message += View->m_sName;
					Message += " defines a document without a name";
					CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
					View->Clear();
					return false;
				}
				// Check if double names
				if(View->m_mDocs.find(DocName)!=View->m_mDocs.end())
				{
					string Message = "The document ";
					Message += DocName;
					Message += " is duplicated in view ";
					Message += View->m_sName;
					CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
					View->Clear();
					return false;
				}
				// Check if double names in header
				if((Header != NULL) && (Header->m_mDocs.find(DocName)!=Header->m_mDocs.end()))
				{
					string Message = "The document ";
					Message += DocName;
					Message += " is define in view ";
					Message += View->m_sName;
					Message += " and its header ";
					Message += Header->m_sName;
					CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
					View->Clear();
					return false;
				}
				// Check if double names in footer
				if((Footer != NULL) && (Footer->m_mDocs.find(DocName)!=Footer->m_mDocs.end()))
				{
					string Message = "The document ";
					Message += DocName;
					Message += " is define in view ";
					Message += View->m_sName;
					Message += " and its footer ";
					Message += Footer->m_sName;
					CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
					View->Clear();
					return false;
				}
				NvNidgetDoc *Doc = new NvNidgetDoc();
				Doc->m_sName = DocName;
				Doc->m_sLoad = DocLoad;
				if(DocType == "dynamic")
					Doc->m_iType = NV_NIDGET_DOC_TYPE_DYNAMIC;
				if(DocType == "static")
					Doc->m_iType = NV_NIDGET_DOC_TYPE_STATIC;
				if(DocType == "inline")
					Doc->m_iType = NV_NIDGET_DOC_TYPE_INLINE;
				View->m_mDocs[DocName] = Doc;
			}
		}
	}

	// If we have header and footer, verify duplicate Docs between them
	if((Header != NULL) && Footer != NULL)
	{
		map<string,NvNidgetDoc*>::iterator posDoc;
		for(posDoc = Header->m_mDocs.begin(); posDoc != Header->m_mDocs.end(); ++posDoc)
		{
			NvNidgetDoc *NidgetDoc = posDoc->second;
			if(Footer->m_mDocs.find(NidgetDoc->m_sName)!=Footer->m_mDocs.end())
			{
				string Message = "The document ";
				Message += NidgetDoc->m_sName;
				Message += " is used both in header ";
				Message += Header->m_sName;
				Message += " and footer ";
				Message += Footer->m_sName;
				Message += "  in view ";
				Message += View->m_sName;
				CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
				View->Clear();
				return false;
			}
		}
	}

	MyTable = (NvObjTable *)(Container.GetObject("forms"));
	if(View->m_iType != NV_NIDGET_VIEW_TYPE_HTML)
	{
		if(MyTable != NULL)
		{
			if(!GetMain()->b_Unicode)
				MyTable->Encode(false, true);
			int NumRows = MyTable->GetNumRow();
			int i;
			for(i=0;i<NumRows;i++)
			{
				string FormName = MyTable->GetCellLine(i, 0);
				CleanString(FormName, true);
				string FormAction = MyTable->GetCellLine(i, 1);
				string FormValidation = MyTable->GetCellLine(i, 2);
				if(FormValidation.length())
				{
					FormAction += "$$$nv_nd_validate_sep$$$";
					FormAction += FormValidation;
				}
				if(FormName.length())
					View->m_mForms[FormName] = FormAction;
			}
		}
	}

	NvObjStringList *MyStringList = (NvObjStringList *)(Container.GetObject("sections"));
	if(View->m_iType != NV_NIDGET_VIEW_TYPE_HTML)
	{
		if(MyStringList != NULL)
		{
			if(!GetMain()->b_Unicode)
				MyStringList->Encode(false, true);
			int i;
			for(i=0;i<MyStringList->GetSize();i++)
				View->m_vSections.push_back(MyStringList->GetValue(i));
		}
	}

	MyTable = (NvObjTable *)(Container.GetObject("nidgets"));
	if(View->m_iType != NV_NIDGET_VIEW_TYPE_HTML)
	{
		if(MyTable != NULL)
		{
			if(!GetMain()->b_Unicode)
				MyTable->Encode(false, true);
			int NumRows = MyTable->GetNumRow();
			int i;
			for(i=0;i<NumRows;i++)
			{
				string NidgetName = MyTable->GetCellLine(i, 0);
				CleanString(NidgetName, true);
				MakeLower(NidgetName);
				// Check no name
				if(!NidgetName.length())
				{
					string Message = "The view ";
					Message += View->m_sName;
					Message += " uses a nidget without a name";
					CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
					View->Clear();
					return false;
				}
				string NidgetClass = MyTable->GetCellLine(i, 1);
				if(!NidgetClass.length())
				{
					string Message = "The nidget ";
					Message += NidgetName;
					Message += " in view ";
					Message += View->m_sName;
					Message += " has no type defined";
					CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
					View->Clear();
					return false;
				}

				// Check if double names
				if(View->m_mNidgets.find(NidgetName)!=View->m_mNidgets.end())
				{
					string Message = "The nidget ";
					Message += NidgetName;
					Message += " is duplicated in view ";
					Message += View->m_sName;
					CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
					View->Clear();
					return false;
				}

				if((Header != NULL) && (Header->m_mNidgets.find(NidgetName)!=Header->m_mNidgets.end()))
				{
					string Message = "The nidget ";
					Message += NidgetName;
					Message += " is used both in view ";
					Message += View->m_sName;
					Message += " and header ";
					Message += Header->m_sName;
					CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
					View->Clear();
					return false;
				}

				if((Footer != NULL) && (Footer->m_mNidgets.find(NidgetName)!=Footer->m_mNidgets.end()))
				{
					string Message = "The nidget ";
					Message += NidgetName;
					Message += " is used both in view ";
					Message += View->m_sName;
					Message += " and footer ";
					Message += Footer->m_sName;
					CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
					View->Clear();
					return false;
				}

				NvNidget *Nidget = new NvNidget();
				Nidget->m_sName = NidgetName;
				Nidget->m_sClass = NidgetClass;
				string NidgetDocs = MyTable->GetCellLine(i, 2);
				while(NidgetDocs.length())
				{
					string DocName = NidgetDocs;
					string::size_type index = NidgetDocs.find(';');
					if(index != string::npos)
					{
						DocName = NidgetDocs.substr(0, index);
						NidgetDocs = NidgetDocs.substr(index+1);
					}
					else
						NidgetDocs = "";
					CleanString(DocName, true);
					MakeLower(DocName);
					if(DocName.length())
					{
						// The document must be declared in the view or its header or its footer
						bool DocFound = false;
						if(View->m_mDocs.find(DocName) != View->m_mDocs.end())
							DocFound = true;
						if((!DocFound) && (Header != NULL) && (Header->m_mDocs.find(DocName)!=Header->m_mDocs.end()))
							DocFound = true;
						if((!DocFound) && (Footer != NULL) && (Footer->m_mDocs.find(DocName)!=Footer->m_mDocs.end()))
							DocFound = true;

						if(!DocFound)
						{
							string Message = "The document ";
							Message += DocName;
							Message += " is used in nidget ";
							Message += NidgetName;
							Message += " but not declared in view";
							Message += View->m_sName;
							CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
							View->Clear();
							return false;
						}
						Nidget->m_vDocs.push_back(DocName);
					}
				}
				View->m_mNidgets[NidgetName] = Nidget;
			}
		}
	}

	// If we have header and footer, verify duplicate Nidgets between them
	if((Header != NULL) && Footer != NULL)
	{
		map<string,NvNidget*>::iterator posNidget;
		for(posNidget = Header->m_mNidgets.begin(); posNidget != Header->m_mNidgets.end(); ++posNidget)
		{
			NvNidget *Nidget = posNidget->second;
			if(Footer->m_mNidgets.find(Nidget->m_sName)!=Footer->m_mNidgets.end())
			{
				string Message = "The nidget ";
				Message += Nidget->m_sName;
				Message += " is used both in header ";
				Message += Header->m_sName;
				Message += " and footer ";
				Message += Footer->m_sName;
				Message += "  in view ";
				Message += View->m_sName;
				CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
				View->Clear();
				return false;
			}
		}
	}

	// Now we must check that there is no duplicated sections
	map<string,string> ForCheckOnly;
	int i;
	for(i=0;i<(int)(View->m_vSections.size());i++)
	{
		string SectionName = View->m_vSections[i];
		CleanString(SectionName, true);
		MakeLower(SectionName);
		if(ForCheckOnly.find(SectionName) != ForCheckOnly.end())
		{
			string Message = "The section ";
			Message += SectionName;
			Message += " is used twice in view ";
			Message += View->m_sName;
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			View->Clear();
			return false;
		}
		ForCheckOnly[SectionName] = SectionName;
	}

	// Now we must check that a form parameter in the view corresponds to a real form name (section name)
	map<string,string>::iterator posForm;
	for(posForm = View->m_mForms.begin(); posForm != View->m_mForms.end(); ++posForm)
	{
		if(ForCheckOnly.find(posForm->first) == ForCheckOnly.end())
		{
			string Message = "The form ";
			Message += posForm->first;
			Message += " is used in the view ";
			Message += View->m_sName;
			Message += " but there is no matching section";
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			View->Clear();
			return false;
		}
	}

	// Now we must check that there is no duplicated sections between header and view
	if(Header != NULL)
	{
		for(i=0;i<(int)(Header->m_vSections.size());i++)
		{
			string SectionName = Header->m_vSections[i];
			CleanString(SectionName, true);
			MakeLower(SectionName);
			if(ForCheckOnly.find(SectionName) != ForCheckOnly.end())
			{
				string Message = "The section ";
				Message += SectionName;
				Message += " is used in view ";
				Message += View->m_sName;
				Message += " and its header ";
				Message += Header->m_sName;
				CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
				View->Clear();
				return false;
			}
			ForCheckOnly[SectionName] = SectionName;
		}
	}

	// Now we must check that there is no duplicated sections between footer and view
	if(Footer != NULL)
	{
		for(i=0;i<(int)(Footer->m_vSections.size());i++)
		{
			string SectionName = Footer->m_vSections[i];
			CleanString(SectionName, true);
			MakeLower(SectionName);
			if(ForCheckOnly.find(SectionName) != ForCheckOnly.end())
			{
				string Message = "The section ";
				Message += SectionName;
				Message += " is used in view ";
				Message += View->m_sName;
				Message += " and its footer ";
				Message += Footer->m_sName;
				CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
				View->Clear();
				return false;
			}
			ForCheckOnly[SectionName] = SectionName;
		}
	}

	ForCheckOnly.clear();

	// Get the sections content
	if(View->m_iType == NV_NIDGET_VIEW_TYPE_HTML)
	{
		if(!Binary.Load(ViewSource.c_str()))
		{
			string Message = "cannot load file:";
			Message += ViewSource;
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			View->Clear();
			return false;
		}

		Buffer = Binary.GetBuffer();
		BufferSize = Binary.m_isize;
		Binary.m_cdata = NULL;
		Binary.m_isize = 0;

		if(Buffer != NULL)
		{
			// Terminates the string with a null character
			char *NewBuffer = (char *)realloc(Buffer, BufferSize+1);
			if(NewBuffer == NULL)
			{
				CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_NOT_ENOUGH_MEMORY);
				if(Buffer != NULL)
					free(Buffer);
				View->Clear();
				return false;
			}
			Buffer = NewBuffer;
			Buffer[BufferSize] = '\0';
			View->m_sSectionsData = Buffer;
			free(Buffer);
			// Keep only the content
			string::size_type index = View->m_sSectionsData.find("<content>");
			if(index != string::npos)
				View->m_sSectionsData = View->m_sSectionsData.substr(index+10);
			index = View->m_sSectionsData.rfind("</content>");
			if(index != string::npos)
				View->m_sSectionsData = View->m_sSectionsData.substr(0, index);
		}
	}
	else
	{
		XsltFile = GetMain()->GetNirvaDir();
		XsltFile += "Files/System/Framework/get_sections.xsl";
		if(!GetNoCaseRealFileName(XsltFile))
		{
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, "get_sections.xsl not found");
			View->Clear();
			return false;
		}

		if(!Binary.Load(ViewSource.c_str()))
		{
			string Message = "cannot load file:";
			Message += ViewSource;
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			View->Clear();
			return false;
		}

		Buffer = Binary.GetBuffer();
		BufferSize = Binary.m_isize;
		Binary.m_cdata = NULL;
		Binary.m_isize = 0;

		// Parse the view for sections
		if(!XmlParser.XsltTransform(&Buffer, &BufferSize, XsltFile.c_str()))
		{
			CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_XSL_PROCESSING_ERROR);
			if(Buffer != NULL)
				free(Buffer);
			View->Clear();
			return false;
		}

		if(Buffer != NULL)
		{
			// Terminates the string with a null character
			char *NewBuffer = (char *)realloc(Buffer, BufferSize+1);
			if(NewBuffer == NULL)
			{
				CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_NOT_ENOUGH_MEMORY);
				if(Buffer != NULL)
					free(Buffer);
				View->Clear();
				return false;
			}
			Buffer = NewBuffer;
			Buffer[BufferSize] = '\0';
			View->m_sSectionsData = Buffer;
			free(Buffer);
			// Keep only the content
			string::size_type index = View->m_sSectionsData.find("<nirva>");
			if(index != string::npos)
				View->m_sSectionsData = View->m_sSectionsData.substr(index+7);
			index = View->m_sSectionsData.rfind("</nirva>");
			if(index != string::npos)
				View->m_sSectionsData = View->m_sSectionsData.substr(0, index);
		}
	}

	// Get the extra head html code
	Binary.Load(ViewSource.c_str());
	Buffer = Binary.GetBuffer();
	BufferSize = Binary.m_isize;
	Binary.m_cdata = NULL;
	Binary.m_isize = 0;
	string Metas = "";
	if(Buffer != NULL)
	{
		// Terminates the string with a null character
		char *NewBuffer = (char *)realloc(Buffer, BufferSize+1);
		if(NewBuffer == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_NOT_ENOUGH_MEMORY);
			if(Buffer != NULL)
				free(Buffer);
			View->Clear();
			return false;
		}
		Buffer = NewBuffer;
		Buffer[BufferSize] = '\0';
		Metas = Buffer;
		free(Buffer);
		// Keep only the content
		string::size_type index = Metas.find("<metas>");
		if(index == string::npos)
			Metas = "";
		else
			Metas = Metas.substr(index+7);
		index = Metas.rfind("</metas>");
		if(index == string::npos)
			Metas = "";
		else
			Metas = Metas.substr(0, index);
	}

	// Create xml files for inline documents
	if(View->m_iType != NV_NIDGET_VIEW_TYPE_HTML)
	{
		XsltFile = GetMain()->GetNirvaDir();
		XsltFile += "Files/System/Framework/get_inlinedocs.xsl";
		if(!GetNoCaseRealFileName(XsltFile))
		{
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, "get_inlinedocs.xsl not found");
			View->Clear();
			return false;
		}

		if(!Binary.Load(ViewSource.c_str()))
		{
			string Message = "cannot load file:";
			Message += ViewSource;
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			View->Clear();
			return false;
		}

		Buffer = Binary.GetBuffer();
		BufferSize = Binary.m_isize;
		Binary.m_cdata = NULL;
		Binary.m_isize = 0;

		// Parse the view for inline docs
		if(!XmlParser.XsltTransform(&Buffer, &BufferSize, XsltFile.c_str()))
		{
			CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_XSL_PROCESSING_ERROR);
			if(Buffer != NULL)
				free(Buffer);
			View->Clear();
			return false;
		}

		string InlineDocs = "";
		if(Buffer != NULL)
		{
			// Terminates the string with a null character
			char *NewBuffer = (char *)realloc(Buffer, BufferSize+1);
			if(NewBuffer == NULL)
			{
				CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_NOT_ENOUGH_MEMORY);
				if(Buffer != NULL)
					free(Buffer);
				View->Clear();
				return false;
			}
			Buffer = NewBuffer;
			Buffer[BufferSize] = '\0';

			
			InlineDocs = Buffer;
			free(Buffer);
			
			// Keep only the content
			string::size_type index = InlineDocs.find("<nirva>");
			if(index != string::npos)
				InlineDocs = InlineDocs.substr(index+7);
			index = InlineDocs.rfind("</nirva>");
			if(index != string::npos)
				InlineDocs = InlineDocs.substr(0, index);
		}

		while(1)
		{
			string::size_type index = InlineDocs.find("<doc_content docname=\"");
			if(index == string::npos)
				break;
			InlineDocs = InlineDocs.substr(index+22);
			index = InlineDocs.find("\">");
			if(index == string::npos)
				break;
			string DocName = InlineDocs.substr(0, index);
			InlineDocs = InlineDocs.substr(index+2);
			CleanString(DocName, true);
			MakeLower(DocName);
			index = InlineDocs.find("</doc_content>");
			if(index == string::npos)
				break;
			string DocContent = InlineDocs.substr(0, index);
			InlineDocs = InlineDocs.substr(index+14);

			if(View->m_mDocs.find(DocName)!=View->m_mDocs.end())
			{
				NvNidgetDoc *NidgetDoc = View->m_mDocs[DocName];
				if(NidgetDoc->m_iType != NV_NIDGET_DOC_TYPE_INLINE)
					continue;
				string InlineDocFileName = WrootDir;
				AddDirSuffix(InlineDocFileName);
				InlineDocFileName += "Framework";
				AddDirSuffix(InlineDocFileName);
				InlineDocFileName += "Inline";
				CreateDir(InlineDocFileName.c_str(), false);
				AddDirSuffix(InlineDocFileName);
				InlineDocFileName += "inlinedoc_";
				InlineDocFileName += View->m_sName;
				InlineDocFileName += "_";
				InlineDocFileName += DocName;
				InlineDocFileName += ".xml";
				if((bDoPublish) || !FileExist(InlineDocFileName.c_str()))
				{
					string InlineDocData = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
					InlineDocData += DocContent;
					RemoveFile(InlineDocFileName.c_str());
					FILE *FileDest;
					FileDest = Nvfopen(InlineDocFileName.c_str(), "wb");
					if(FileDest == NULL)
					{
						string Message = "cannot create file:";
						Message += InlineDocFileName;
						CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
						View->Clear();
						return false;
					}
					if(fwrite(InlineDocData.c_str(), 1, InlineDocData.length(), FileDest)  != InlineDocData.length())
					{
						fclose(FileDest);
						string Message = "cannot write file:";
						Message += InlineDocFileName;
						CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
						View->Clear();
						return false;
					}
					fclose(FileDest);
					NidgetDoc->m_sLoad = "Inline/";
					NidgetDoc->m_sLoad += "inlinedoc_";
					NidgetDoc->m_sLoad += View->m_sName;
					NidgetDoc->m_sLoad += "_";
					NidgetDoc->m_sLoad += DocName;
					NidgetDoc->m_sLoad += ".xml";
				}
			}
		}
	}

	// Prepare xml data
	string Data = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
	Data += "<view>\n";
	Data += "<name>";
	Data += View->m_sName;
	Data += "</name>\n";
	Data += "<title>";
	Data += View->m_sTitle;
	Data += "</title>\n";
	if(Metas.length())
	{
		Data += "<metas>\n";
		Data += Metas;
		Data += "</metas>\n";
	}
	if(m_sIcon.length())
	{
		Data += "<icon>";
		Data += m_sIcon;
		Data += "</icon>\n";
		string::size_type index = m_sIcon.rfind('.');
		string extension = "png";
		if(index != string::npos)
			extension = m_sIcon.substr(index+1);
		MakeLower(extension);
		Data += "<icontype>";
		Data += "image/";
		Data += extension;
		Data += "</icontype>\n";
	}
	if(m_sTheme.length())
	{
		Data += "<theme>";
		Data += m_sTheme;
		if(m_sTheme.rfind(".css") == string::npos)
			Data += ".css";
		Data += "</theme>\n";
	}
	Data += "<language>";
	Data += View->m_sLanguage;
	Data += "</language>\n";
	Data += "<type>";
	if(View->m_iType == NV_NIDGET_VIEW_TYPE_STATIC)
		Data += "STATIC";
	else if(View->m_iType == NV_NIDGET_VIEW_TYPE_HTML)
		Data += "HTML";
	else
		Data += "DYNAMIC";
	Data += "</type>\n";
	Data += "<debug>";
	Data += m_sDebugLevel;
	Data += "</debug>\n";
	Data += "<includes>\n";
	map<string,string>::iterator pos;
	for(pos = m_mIncludes.begin(); pos != m_mIncludes.end(); ++pos)
	{
		Data += "<include>Framework/Includes/";
		Data += pos->first;
		Data += "</include>";
	}
	for(pos = m_mPackages.begin(); pos != m_mPackages.end(); ++pos)
	{
		Data += "<include>Framework/Nidgets/";
		Data += pos->first;
		Data += "</include>";
	}
	Data += "</includes>\n";
	Data += "<documents>\n";
	map<string,NvNidgetDoc*>::iterator posDoc;
	for(posDoc = View->m_mDocs.begin(); posDoc != View->m_mDocs.end(); ++posDoc)
	{
		NvNidgetDoc *NidgetDoc = posDoc->second;
		int DocType = NidgetDoc->m_iType;
		Data += "<document>";
		Data += "<name>";
		Data += NidgetDoc->m_sName;
		Data += "</name>";
		Data += "<type>";
		if(DocType == NV_NIDGET_DOC_TYPE_DYNAMIC)
			Data += "DYNAMIC";
		else
			Data += "STATIC";
		Data += "</type>";
		Data += "<load>";
		Data += NidgetDoc->m_sLoad;
		Data += "</load>";
		Data += "</document>";
	}
	if(Header != NULL)
	{
		for(posDoc = Header->m_mDocs.begin(); posDoc != Header->m_mDocs.end(); ++posDoc)
		{
			NvNidgetDoc *NidgetDoc = posDoc->second;
			int DocType = NidgetDoc->m_iType;
			Data += "<hdocument>";
			Data += "<name>";
			Data += NidgetDoc->m_sName;
			Data += "</name>";
			Data += "<type>";
			if(DocType == NV_NIDGET_DOC_TYPE_DYNAMIC)
				Data += "DYNAMIC";
			else
				Data += "STATIC";
			Data += "</type>";
			Data += "<load>";
			Data += NidgetDoc->m_sLoad;
			Data += "</load>";
			Data += "</hdocument>";
		}
	}
	if(Footer != NULL)
	{
		for(posDoc = Footer->m_mDocs.begin(); posDoc != Footer->m_mDocs.end(); ++posDoc)
		{
			NvNidgetDoc *NidgetDoc = posDoc->second;
			int DocType = NidgetDoc->m_iType;
			Data += "<fdocument>";
			Data += "<name>";
			Data += NidgetDoc->m_sName;
			Data += "</name>";
			Data += "<type>";
			if(DocType == NV_NIDGET_DOC_TYPE_DYNAMIC)
				Data += "DYNAMIC";
			else
				Data += "STATIC";
			Data += "</type>";
			Data += "<load>";
			Data += NidgetDoc->m_sLoad;
			Data += "</load>";
			Data += "</fdocument>";
		}
	}
	Data += "</documents>\n";
	Data += "<forms>\n";
	for(posForm = View->m_mForms.begin(); posForm != View->m_mForms.end(); ++posForm)
	{
		Data += "<form>";
		Data += "<name>";
		Data += posForm->first;
		Data += "</name>";
		Data += "<action>";
		string FormAction = posForm->second;
		string FormValidate = "";
		string::size_type index = FormAction.find("$$$nv_nd_validate_sep$$$");
		if(index != string::npos)
		{
			FormValidate = FormAction.substr(index + 24);
			FormAction = FormAction.substr(0, index);
		}
		Data += FormAction;
		Data += "</action>";
		Data += "<validation>";
		Data += FormValidate;
		Data += "</validation>";
		Data += "</form>";
	}
	if(Header != NULL)
	{
		for(posForm = Header->m_mForms.begin(); posForm != Header->m_mForms.end(); ++posForm)
		{
			Data += "<hform>";
			Data += "<name>";
			Data += posForm->first;
			Data += "</name>";
			Data += "<action>";
			string FormAction = posForm->second;
			string FormValidate = "";
			string::size_type index = FormAction.find("$$$nv_nd_validate_sep$$$");
			if(index != string::npos)
			{
				FormValidate = FormAction.substr(index + 24);
				FormAction = FormAction.substr(0, index);
			}
			Data += FormAction;
			Data += "</action>";
			Data += "<validation>";
			Data += FormValidate;
			Data += "</validation>";
			Data += "</hform>";
		}
	}
	if(Footer != NULL)
	{
		for(posForm = Footer->m_mForms.begin(); posForm != Footer->m_mForms.end(); ++posForm)
		{
			Data += "<fform>";
			Data += "<name>";
			Data += posForm->first;
			Data += "</name>";
			Data += "<action>";
			string FormAction = posForm->second;
			string FormValidate = "";
			string::size_type index = FormAction.find("$$$nv_nd_validate_sep$$$");
			if(index != string::npos)
			{
				FormValidate = FormAction.substr(index + 24);
				FormAction = FormAction.substr(0, index);
			}
			Data += FormAction;
			Data += "</action>";
			Data += "<validation>";
			Data += FormValidate;
			Data += "</validation>";
			Data += "</fform>";
		}
	}
	Data += "</forms>\n";
	Data += "<nidgets>\n";
	map<string,NvNidget*>::iterator posNidget;
	for(posNidget = View->m_mNidgets.begin(); posNidget != View->m_mNidgets.end(); ++posNidget)
	{
		NvNidget *Nidget = posNidget->second;
		Data += "<nidget>";
		Data += "<name>";
		Data += Nidget->m_sName;
		Data += "</name>";
		Data += "<classname>";
		Data += Nidget->m_sClass;
		Data += "</classname>";
		Data += "<documents>";
		for(i=0;i<(int)(Nidget->m_vDocs.size());i++)
		{
			Data += "<document>";
			Data += Nidget->m_vDocs[i];
			Data += "</document>";
		}
		Data += "</documents>";
		Data += "</nidget>";
	}
	if(Header != NULL)
	{
		for(posNidget = Header->m_mNidgets.begin(); posNidget != Header->m_mNidgets.end(); ++posNidget)
		{
			NvNidget *Nidget = posNidget->second;
			Data += "<hnidget>";
			Data += "<name>";
			Data += Nidget->m_sName;
			Data += "</name>";
			Data += "<classname>";
			Data += Nidget->m_sClass;
			Data += "</classname>";
			Data += "<documents>";
			for(i=0;i<(int)(Nidget->m_vDocs.size());i++)
			{
				Data += "<document>";
				Data += Nidget->m_vDocs[i];
				Data += "</document>";
			}
			Data += "</documents>";
			Data += "</hnidget>";
		}
	}
	if(Footer != NULL)
	{
		for(posNidget = Footer->m_mNidgets.begin(); posNidget != Footer->m_mNidgets.end(); ++posNidget)
		{
			NvNidget *Nidget = posNidget->second;
			Data += "<fnidget>";
			Data += "<name>";
			Data += Nidget->m_sName;
			Data += "</name>";
			Data += "<classname>";
			Data += Nidget->m_sClass;
			Data += "</classname>";
			Data += "<documents>";
			for(i=0;i<(int)(Nidget->m_vDocs.size());i++)
			{
				Data += "<document>";
				Data += Nidget->m_vDocs[i];
				Data += "</document>";
			}
			Data += "</documents>";
			Data += "</fnidget>";
		}
	}
	Data += "</nidgets>\n";
	Data += "<getparams>\n";
	Data += "<paramfile>";
	if(View->m_sParameters.length())
	{
		Data += "Framework/Params/";
		Data += View->m_sName;
		Data += "_params_NVLANGUAGE";
		Data += ".xml";
	}
	Data += "</paramfile>";
	if(Header != NULL)
	{
		Data += "<hparamfile>";
		if(Header->m_sParameters.length())
		{
			Data += "Framework/Params/";
			Data += Header->m_sName;
			Data += "_params_NVLANGUAGE";
			Data += ".xml";
		}
		Data += "</hparamfile>";
	}
	if(Footer != NULL)
	{
		Data += "<fparamfile>";
		if(Footer->m_sParameters.length())
		{
			Data += "Framework/Params/";
			Data += Footer->m_sName;
			Data += "_params_NVLANGUAGE";
			Data += ".xml";
		}
		Data += "</fparamfile>";
	}
	Data += "</getparams>\n";
	Data += "<parseparams>\n";
	for(posNidget = View->m_mNidgets.begin(); posNidget != View->m_mNidgets.end(); ++posNidget)
	{
		NvNidget *Nidget = posNidget->second;
		Data += "<nidget>";
		Data += Nidget->m_sName;
		Data += "</nidget>";
	}
	if(Header != NULL)
	{
		for(posNidget = Header->m_mNidgets.begin(); posNidget != Header->m_mNidgets.end(); ++posNidget)
		{
			NvNidget *Nidget = posNidget->second;
			Data += "<hnidget>";
			Data += Nidget->m_sName;
			Data += "</hnidget>";
		}
	}
	if(Footer != NULL)
	{
		for(posNidget = Footer->m_mNidgets.begin(); posNidget != Footer->m_mNidgets.end(); ++posNidget)
		{
			NvNidget *Nidget = posNidget->second;
			Data += "<fnidget>";
			Data += Nidget->m_sName;
			Data += "</fnidget>";
		}
	}
	Data += "</parseparams>\n";
	if(View->m_iType != NV_NIDGET_VIEW_TYPE_HTML)
	{
		Data += "<body class=\"";
		Data += View->m_sClass;
		Data += "\">\n";
		if(Header != NULL)
			Data += Header->m_sSectionsData;
		Data += View->m_sSectionsData;
		if(Footer != NULL)
			Data += Footer->m_sSectionsData;
		Data += "</body>\n";
	}
	Data += "</view>\n";

	// Build the view specific javascript
	XsltFile = GetMain()->GetNirvaDir();
	XsltFile += "Files/System/Framework/build_viewjs.xsl";
	if(!GetNoCaseRealFileName(XsltFile))
	{
		CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, "build_viewjs.xsl not found");
		View->Clear();
		return false;
	}
	
	// Parse the javascript view
	string DataViewJs = Data;
	if(!XmlParser.XsltTransform(DataViewJs, XsltFile))
	{
		CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_XSL_PROCESSING_ERROR);
		View->Clear();
		return false;
	}

	// Publish the javascript view
	string ViewJsFile = WrootDir;
	AddDirSuffix(ViewJsFile);
	ViewJsFile += "Framework";
	AddDirSuffix(ViewJsFile);
	ViewJsFile += "Views";
	CreateDir(ViewJsFile.c_str(), false);
	AddDirSuffix(ViewJsFile);
	ViewJsFile += View->m_sName;
	ViewJsFile += ".js";
	if((bDoPublish) || !FileExist(ViewJsFile.c_str()))
	{
		RemoveFile(ViewJsFile.c_str());
		FILE *FileDest;
		FileDest = Nvfopen(ViewJsFile.c_str(), "wb");
		if(FileDest == NULL)
		{
			string Message = "cannot create file:";
			Message += ViewJsFile;
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			View->Clear();
			return false;
		}
		if(fwrite(DataViewJs.c_str(), 1, DataViewJs.length(), FileDest)  != DataViewJs.length())
		{
			fclose(FileDest);
			string Message = "cannot write file:";
			Message += ViewJsFile;
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			View->Clear();
			return false;
		}
		fclose(FileDest);
	}
	DataViewJs = "";

	XsltFile = GetMain()->GetNirvaDir();
	XsltFile += "Files/System/Framework/build_view.xsl";
	if(!GetNoCaseRealFileName(XsltFile))
	{
		CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, "build_view.xsl not found");
		View->Clear();
		return false;
	}
	
	// Parse the view
	if(!XmlParser.XsltTransform(Data, XsltFile))
	{
		CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_XSL_PROCESSING_ERROR);
		View->Clear();
		return false;
	}

	// For HTML view, insert the content here
	if(View->m_iType == NV_NIDGET_VIEW_TYPE_HTML)
	{
		string::size_type index=Data.rfind("</html>");
		if(index != string::npos)
		{
			Data.erase(index);
			Data += View->m_sSectionsData;
			Data += "</html>";
		}
	}


	// Prepare for future resolving of language
	string::size_type index=Data.find("$$$NV$$$LANGUAGE$$$");
	if(index != string::npos)
	{
		View->LanguageIndex = (int)index;
		Data.replace(index, 19, "");
	}

	// Prepare for future resolving of session ID
	index=Data.find("$$$NV$$$SESSIONID$$$");
	if(index != string::npos)
	{
		View->SessionIdIndex = (int)index;
		Data.replace(index, 20, "");
	}

	// Resolve the application name
	index=Data.find("$$$NV$$$APPLICATION$$$");
	if(index != string::npos)
		Data.replace(index, 22, m_sAppName);
	// Now the view data is ok, we keep it for performance
	View->m_sViewData = Data;

	return true;
}



bool NvNidgetFramework::CheckViewParameters(string& ViewName, string& Language)
{
	string WrootDir = GetMain()->GetNirvaDir();
	AddDirSuffix(WrootDir);
	WrootDir += "Applications";
	AddDirSuffix(WrootDir);
	WrootDir += m_sAppName;
	AddDirSuffix(WrootDir);
	WrootDir += "Wroot";
	string FileName = WrootDir;
	AddDirSuffix(FileName);
	FileName += "Framework";
	AddDirSuffix(FileName);
	FileName += "Params";
	AddDirSuffix(FileName);
	FileName += ViewName;
	FileName += "_params";
	if(Language.length())
	{
		FileName += "_";
		FileName += Language;
	}
	FileName += ".xml";
	if(FileExist(FileName.c_str()))
		return false;

	return true;
}


bool NvNidgetFramework::BuildViewParameters(NvNidgetView* View, NvCommand *CommandToExecute, string Language)
{
	if(!View->m_sParameters.length())
		return true;	// Nothing to do if no parameters defined

	string WrootDir = GetMain()->GetNirvaDir();
	AddDirSuffix(WrootDir);
	WrootDir += "Applications";
	AddDirSuffix(WrootDir);
	WrootDir += m_sAppName;
	AddDirSuffix(WrootDir);
	WrootDir += "Wroot";

	string PublishFileName = WrootDir;
	AddDirSuffix(PublishFileName);
	PublishFileName += "Framework";
	AddDirSuffix(PublishFileName);
	PublishFileName += "Params";
	if(!CreateDir(PublishFileName.c_str(), false))	// Create the dir if it doesn't exist
	{
		string Message = "cannot create directory:";
		Message += PublishFileName;
		CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
		View->Clear();
		return false;
	}
	AddDirSuffix(PublishFileName);
	PublishFileName += View->m_sName;
	PublishFileName += "_params";
	PublishFileName += "_";
	PublishFileName += Language;
	PublishFileName += ".xml";

	// Check source param file
	// Set the Framework source directory if not yet done
	string SourceFileName = GetFrameworkSourceDir();
	AddDirSuffix(SourceFileName);
	SourceFileName += "Params";
	AddDirSuffix(SourceFileName);
	SourceFileName += View->m_sParameters;
	if(!FileExist(SourceFileName.c_str()))
	{
		string Message = "The file ";
		Message += SourceFileName;
		Message += " used in view ";
		Message += View->m_sName;
		Message += " doesn't exist";
		CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
		View->Clear();
		return false;
	}
	
	bool bApplyLabels = true;
	if(!View->m_sLabels.length())
		bApplyLabels = false;

	// PM TODO Apply labels
	NvContainer Container;
	NvObjIndStringList *Labels = NULL;
	NvObjBinary Binary;
	if(bApplyLabels)
	{
		string LabelFileName = GetFrameworkSourceDir();
		AddDirSuffix(LabelFileName);
		LabelFileName += "Labels";
		AddDirSuffix(LabelFileName);
		LabelFileName += View->m_sLabels;
		if(!FileExist(LabelFileName.c_str()))
		{
			string Message = "The file ";
			Message += SourceFileName;
			Message += " used in view ";
			Message += View->m_sName;
			Message += " doesn't exist";
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			View->Clear();
			return false;
		}

		// Then load the label
		NvXml XmlParser;
		string XsltFile = GetMain()->GetNirvaDir();
		XsltFile += "Files/System/Framework/get_labels.xsl";
		if(!GetNoCaseRealFileName(XsltFile))
		{
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, "get_labels.xsl not found");
			View->Clear();
			return false;
		}

		string NameSpace;
		int XmlCoding = XML_MODEL_SIMPLE;
		string LabelSource = GetFrameworkSourceDir();
		AddDirSuffix(LabelSource);
		LabelSource += "Labels";
		AddDirSuffix(LabelSource);
		LabelSource += View->m_sLabels;

		if(!GetNoCaseRealFileName(LabelSource))
		{
			string Message = "file not found:";
			Message += LabelSource;
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			View->Clear();
			return false;
		}

		if(!Binary.Load(LabelSource.c_str()))
		{
			string Message = "cannot load file:";
			Message += LabelSource;
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			View->Clear();
			return false;
		}

		char *Buffer = Binary.GetBuffer();
		unsigned long BufferSize = Binary.m_isize;
		Binary.m_cdata = NULL;
		Binary.m_isize = 0;

		// Parse the view
		if(!XmlParser.XsltTransform(&Buffer, &BufferSize, XsltFile.c_str()))
		{
			CommandToExecute->Error.SetError("SYSTEM", "XML", NV_XML_XSL_PROCESSING_ERROR);
			if(Buffer != NULL)
				free(Buffer);
			View->Clear();
			return false;
		}

		// Get input
		if(!XmlParser.SetInputData(&Container, Buffer, BufferSize, CommandToExecute, NameSpace, XmlCoding))
		{
			string Message = "cannot get label data from file:";
			Message += LabelSource;
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			if(Buffer != NULL)
				free(Buffer);
			View->Clear();
			return false;
		}
		
		if(Buffer != NULL)
			free(Buffer);

		Labels = (NvObjIndStringList *)(Container.GetObject(Language.c_str()));
		if((Labels == NULL) || (!Labels->GetSize())) 
			bApplyLabels = false;
	}

	if(bApplyLabels)
	{
		// First Load the parameters data
		if(!Binary.Load(SourceFileName.c_str()))
		{
			string Message = "cannot load file:";
			Message += SourceFileName.c_str();
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			View->Clear();
			return false;
		}

		char *Buffer = Binary.GetBuffer();
		unsigned long BufferSize = Binary.m_isize;
		Binary.m_cdata = NULL;
		Binary.m_isize = 0;
		char *NewBuffer = (char *)realloc(Buffer, BufferSize+1);
		if(NewBuffer == NULL)
		{
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_NOT_ENOUGH_MEMORY);
			if(Buffer != NULL)
				free(Buffer);
			View->Clear();
			return false;
		}
		Buffer = NewBuffer;
		Buffer[BufferSize] = '\0';

		string ParametersData = (const char *)(Buffer);
		if(Buffer != NULL)
			free(Buffer);

		// If we are not running unicode, we must encode the container to utf-8
		if(!GetMain()->b_Unicode)
			Labels->Encode(false, true);
		map<string,string>::iterator pos;
		string::size_type index;
		for(pos = Labels->m_mList.begin(); pos != Labels->m_mList.end(); ++pos)
		{
			string LabelToReplace = "[NV_LABEL:";
			LabelToReplace += pos->first;
			LabelToReplace += "]";
			while((index=ParametersData.find(LabelToReplace)) != string::npos)
				ParametersData.replace(index, LabelToReplace.length(), pos->second);
		}
		Binary.Append(ParametersData);
		if(!Binary.Save(PublishFileName.c_str()))
		{
			string Message = "cannot create the file:";
			Message += PublishFileName;
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			View->Clear();
			return false;
		}
		Binary.Clear();

	}
	else
	{
		if(!CopyFileEx(SourceFileName.c_str(), PublishFileName.c_str()))
		{
			string Message = "cannot create the file:";
			Message += PublishFileName;
			CommandToExecute->Error.SetError("SYSTEM", "NIDGETS", NV_NIDGET_CANNOT_BUILD_VIEW, Message.c_str());
			View->Clear();
			return false;
		}
	}

	return true;
}

string& NvNidgetFramework::GetFrameworkSourceDir()
{
	if(!m_sFrameworkSourceDir.length())
	{
		m_sFrameworkSourceDir = GetMain()->GetNirvaDir();
		AddDirSuffix(m_sFrameworkSourceDir);
		m_sFrameworkSourceDir += "Applications";
		AddDirSuffix(m_sFrameworkSourceDir);
		m_sFrameworkSourceDir += m_sAppName;
		AddDirSuffix(m_sFrameworkSourceDir);
		m_sFrameworkSourceDir += "Files";
		AddDirSuffix(m_sFrameworkSourceDir);
		m_sFrameworkSourceDir += "Framework";
	}
	return m_sFrameworkSourceDir;
}

bool NvNidgetFramework::Skeleton()
{
	string FrameworkDstDir = GetFrameworkSourceDir();
	if(!CreateDir(FrameworkDstDir.c_str()))
		return false;
	string ConfigFile = FrameworkDstDir;
	AddDirSuffix(ConfigFile);
	ConfigFile += "config.xml";
	if(!FileExist(ConfigFile.c_str()))
	{
		// The framework doesn't exist on the target application

		// Copy the default framework from File/Framework
		string FrameworkSrcDir = GetMain()->GetNirvaDir();
		FrameworkSrcDir += "Files/Framework";
		if(!CopyDir(FrameworkSrcDir.c_str(), FrameworkDstDir.c_str()))
			return false;
	}

	// Copy the ajax connector
	string SourceFile = GetMain()->GetNirvaDir();
	SourceFile += "Sdk/Connectors/ajax/nvajax.js";
	if(!GetNoCaseRealFileName(SourceFile))
		return false;

	string DestinationFile = FrameworkDstDir;
	AddDirSuffix(DestinationFile);
	DestinationFile += "System";
	AddDirSuffix(DestinationFile);
	DestinationFile += "nvajax.js";
	if(!GetNoCaseRealFileName(DestinationFile))
	{
		if(!CopyFileEx(SourceFile.c_str(), DestinationFile.c_str()))
			return false;
	}

	string WrootDir = GetMain()->GetNirvaDir();
	AddDirSuffix(WrootDir);
	WrootDir += "Applications";
	AddDirSuffix(WrootDir);
	WrootDir += m_sAppName;
	AddDirSuffix(WrootDir);
	WrootDir += "Wroot";
	AddDirSuffix(WrootDir);
	WrootDir += "Framework";
	if(!CreateDir(WrootDir.c_str()))
		return false;

	return true;
}



/////////////////////////////////////////////////////////////////////////////
// NvJspCleaner class
NvJspCleaner::NvJspCleaner()
{
	theLookahead = EOF;
}

NvJspCleaner::~NvJspCleaner()
{
}

// isAlphanum -- return true if the character is a letter, digit, underscore,
//        dollar sign, or non-ASCII character.
int NvJspCleaner::isAlphanum(int c)
{
    return ((c >= 'a' && c <= 'z') || (c >= '0' && c <= '9') ||
        (c >= 'A' && c <= 'Z') || c == '_' || c == '$' || c == '\\' ||
        c > 126);
}


// get -- return the next character from FileSource. Watch out for lookahead. If
//        the character is a control character, translate it to a space or
//        linefeed.
int NvJspCleaner::get()
{
    int c = theLookahead;
    theLookahead = EOF;
    if (c == EOF) {
        c = getc(FileSource);
    }
    if (c >= ' ' || c == '\n' || c == EOF) {
        return c;
    }
    if (c == '\r') {
        return '\n';
    }
    return ' ';
}


// peek -- get the next character without getting it.
int NvJspCleaner::peek()
{
    theLookahead = get();
    return theLookahead;
}


// next -- get the next character, excluding comments. peek() is used to see
//        if a '/' is followed by a '/' or '*'.
int NvJspCleaner::next()
{
    int c = get();
    if  (c == '/') {
        switch (peek()) {
        case '/':
            for (;;) {
                c = get();
                if (c <= '\n') {
                    return c;
                }
            }
        case '*':
            get();
            for (;;) {
                switch (get()) {
                case '*':
                    if (peek() == '/') {
                        get();
                        return ' ';
                    }
                    break;
                case EOF:
                    fprintf(stderr, "Error: JSMIN Unterminated comment.\n");
                    exit(1);
                }
            }
        default:
            return c;
        }
    }
    return c;
}


// action -- do something! What you do is determined by the argument:
//        1   Output A. Copy B to A. Get the next B.
//        2   Copy B to A. Get the next B. (Delete A).
//        3   Get the next B. (Delete B).
//   action treats a string as a single character. Wow!
//   action recognizes a regular expression if it is preceded by ( or , or =.
bool NvJspCleaner::action(int d)
{
    switch (d) {
    case 1:
        putc(theA, FileDest);
    case 2:
        theA = theB;
        if (theA == '\'' || theA == '"') {
            for (;;) {
                putc(theA, FileDest);
                theA = get();
                if (theA == theB) {
                    break;
                }
                if (theA == '\\') {
                    putc(theA, FileDest);
                    theA = get();
                }
                if (theA == EOF) {
					return false;	// unterminated string literal
                }
            }
        }
    case 3:
        theB = next();
        if (theB == '/' && (theA == '(' || theA == ',' || theA == '=' ||
                            theA == ':' || theA == '[' || theA == '!' ||
                            theA == '&' || theA == '|' || theA == '?' ||
                            theA == '{' || theA == '}' || theA == ';' ||
                            theA == '\n')) {
            putc(theA, FileDest);
            putc(theB, FileDest);
            for (;;) {
                theA = get();
                if (theA == '/') {
                    break;
                }
                if (theA =='\\') {
                    putc(theA, FileDest);
                    theA = get();
                }
                if (theA == EOF) {
					return false;	// unterminated Regular Expression literal
                }
                putc(theA, FileDest);
            }
            theB = next();
        }
    }
	return true;
}


// jsmin -- Copy the input to the output, deleting the characters which are
//        insignificant to JavaScript. Comments will be removed. Tabs will be
//        replaced with spaces. Carriage returns will be replaced with linefeeds.
//        Most spaces and linefeeds will be removed.
bool NvJspCleaner::jsmin()
{
    theA = '\n';
    if(!action(3))
		return false;
    while (theA != EOF) {
        switch (theA) {
        case ' ':
            if (isAlphanum(theB)) {
                if(!action(1))
					return false;
            } else {
                if(!action(2))
					return false;
            }
            break;
        case '\n':
            switch (theB) {
            case '{':
            case '[':
            case '(':
            case '+':
            case '-':
				if(!action(1))
					return false;
                break;
            case ' ':
                if(!action(3))
					return false;
                break;
            default:
                if (isAlphanum(theB)) {
                    if(!action(1))
						return false;
                } else {
                    if(!action(2))
						return false;
                }
            }
            break;
        default:
            switch (theB) {
            case ' ':
                if (isAlphanum(theA)) {
                    if(!action(1))
						return false;
                    break;
                }
                if(!action(3))
					return false;
                break;
            case '\n':
                switch (theA) {
                case '}':
                case ']':
                case ')':
                case '+':
                case '-':
                case '"':
                case '\'':
                    if(!action(1))
						return false;
                    break;
                default:
                    if (isAlphanum(theA)) {
                        if(!action(1))
							return false;
                    } else {
                        if(!action(3))
							return false;
                    }
                }
                break;
            default:
                if(!action(1))
					return false;
                break;
            }
        }
    }

	return true;
}

// DoClean -- Clean the source file and write the result into the destination file.
bool NvJspCleaner::DoClean(const char* SrcFile, const char* DstFile)
{
	FileSource = Nvfopen(SrcFile, "rb");
	if(FileSource == NULL)
		return false;

	FileDest = Nvfopen(DstFile, "wb");
	if(FileDest == NULL)
	{
		fclose(FileSource);
		return false;
	}

	bool result = jsmin();

	fclose(FileDest);
	fclose(FileSource);

	return result;
}

