// ***************************************************************************************
// File: nvsext.h
// Description: Nirva external service header.
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifndef _NVSEXT_H
#define _NVSEXT_H



// Classes declared in this file
class NvServiceCommand;
class NvRendererInt;

#ifndef WIN32
#include <pthread.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#ifdef WIN32
#include <windows.h>
#include <io.h>
#else
#include <unistd.h>
#include <ctype.h>
#endif


// Command source
#define NV_SOURCE_CLIENT				0		// nvc source
#define NV_SOURCE_BROWSER				1		// web browser source
#define NV_SOURCE_PROCEDURE				2		// procedure source
#define NV_SOURCE_SERVICE				3		// service source
#define NV_SOURCE_SERVER				4		// server source (used in init and close of services)




/////////////////////////////////////////////////////////////////////////////
// NvServiceCommand class
// Interface to services
class NvServiceCommand
{
public:
	virtual void SetError(const char *ErrorClass = "", int ErrorCode = 0, const char *ErrorInfo = "") = 0 ;
	virtual bool Command(const char *CommandString, char *Buffer = NULL, int BufferSize = 0, int *DataSize = NULL) = 0 ;
	virtual void GetSessionId(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetClass(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetCommand(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetInContainer(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetOutContainer(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetLanguage(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetMachineName(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetMachineUser(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetError(const char *InfoType, char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual int GetNumParameters() = 0 ;
	virtual int GetSource() = 0 ;
	virtual bool ParameterExist(const char *ParameterName) = 0 ;
	virtual void GetParameter(const char *ParameterName, char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetParameterName(int index, char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void AddHttpHeader(const char *HeaderName, const char *HeaderValue) = 0 ;
	virtual void SetServiceSession(void *ServiceSession) = 0 ;
	virtual void* GetServiceSession() = 0 ;
	virtual bool IsCommand(const char *Class = NULL, const char *Command = NULL, int Source = -1) = 0 ;
};


// Renderer interface
class NvRendererInt
{
public:
	virtual const char* GetInputData() = 0 ;
	virtual unsigned long GetInputDataSize() = 0 ;
	virtual const char* GetInputEncoding() = 0 ;
	virtual const char* GetPageName() = 0 ;
	virtual char* AllocOutputBuffer(unsigned long Size) = 0 ;
	virtual char* GetOutputBuffer() = 0 ;
	virtual unsigned long GetOutputBufferSize() = 0 ;
	virtual const char* GetSessionId() = 0 ;
	virtual const char* GetAppName() = 0 ;
	virtual void SetReason(const char* Reason) = 0 ;
	virtual void SetContentType(const char* ContentType) = 0 ;
	virtual void* GetExtraData(const char* DataName) = 0 ;
	virtual void FreeInputData() = 0 ;
};

#endif //_NVSEXT_H
