// ***************************************************************************************
// File: perlint.h
// Description: Interface to perl.
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifndef _PERLINT_H
#define _PERLINT_H

#include "task.h"
#include "util.h"
#include "../nvperl/eperl.h"


// Classes declared in this file
class NvPerl;
class NvPerlScript;

// NvPerl class. Interface to perl. There is only one instance of this class in the 

class NvPerl
{
public:

	NvMutex ProtectPerl;						// Mutex for perl interface protection
	HMODULE hLibPerl;							// Handle to the nvperl dynamic library
	PERLGETINTERPRETOR PerlLibGetInterpretor;		// Perl library GetInterpretor function pointer
	PERLFREEINTERPRETOR PerlLibFreeInterpretor;		// Perl library FreeInterpretor function pointer
	PERLRUNSCRIPT PerlLibRunScript;					// Perl library RunScript function pointer
	PERLRUNSCRIPTMEMORY PerlLibRunScriptMemory;		// Perl library RunScript function pointer
	INITPERLLIB PerlInitLib;						// Perl library InitLib function pointer
	EXITPERLLIB PerlExitLib;						// Perl library ExitLib function pointer
	int NumInstances;


	NvPerl();
	virtual ~NvPerl();

	bool LoadPerlLib();
	void FreePerlLib();
	bool IsPerlLibLoaded();
	NVPERLINTERPRETOR PerlGetInterpretor();
	void PerlFreeInterpretor(NVPERLINTERPRETOR Interpretor);
	int PerlRunScript(NVPERLINTERPRETOR Interpretor, const char* ScriptFileName, void *pCommand);
	int PerlThreadRunScript(const char* ScriptFileName, void *pCommand, bool SameThread = false);
	void AutoFree();
	char* GetScriptData(const char* ScriptFileName);
};


// Perl script instance
class NvPerlSript : public NvTask
{
// Attributes
public:
	const char* ScriptFile;
	void *pCommandHere;
	NVPERLINTERPRETOR PerlInterpretor;
	bool b_Initialized;

	NvPerlSript();

	// Overridable callbacks
protected:
	virtual int Main();
	virtual bool OnInit();
	virtual void OnCleanup();

public:
	int RunScript(const char* ScriptFileName, void *pCommand, bool SameThread = false);
};



#endif //_PERLINT_H
